@@ -52,7 +52,9 @@
 #include <string.h>
 
 #include "gstflacdec.h"
+#include <gst/gst-i18n-plugin.h>
 #include <gst/gsttagsetter.h>
+#include <gst/base/gsttypefindhelper.h>
 
 #include <gst/tag/tag.h>
 
@@ -85,14 +87,13 @@ static gboolean gst_flac_dec_sink_activate_pull (GstPad * sinkpad,
     gboolean active);
 static gboolean gst_flac_dec_sink_activate_push (GstPad * sinkpad,
     gboolean active);
-static void gst_flac_dec_send_newsegment (GstFlacDec * flacdec,
-    gboolean update);
 static gboolean gst_flac_dec_sink_event (GstPad * pad, GstEvent * event);
 static GstFlowReturn gst_flac_dec_chain (GstPad * pad, GstBuffer * buf);
 static void gst_flac_dec_reset_decoders (GstFlacDec * flacdec);
 static void gst_flac_dec_setup_seekable_decoder (GstFlacDec * flacdec);
 static void gst_flac_dec_setup_stream_decoder (GstFlacDec * flacdec);
 
+#ifdef LEGACY_FLAC
 static FLAC__SeekableStreamDecoderReadStatus
 gst_flac_dec_read_seekable (const FLAC__SeekableStreamDecoder * decoder,
     FLAC__byte buffer[], unsigned *bytes, void *client_data);
@@ -130,6 +131,33 @@ static void gst_flac_dec_error_callback_seekable (const
     FLAC__StreamDecoderErrorStatus status, void *client_data);
 static void gst_flac_dec_error_callback_stream (const FLAC__StreamDecoder *
     decoder, FLAC__StreamDecoderErrorStatus status, void *client_data);
+#else
+static FLAC__StreamDecoderReadStatus
+gst_flac_dec_read_seekable (const FLAC__StreamDecoder * decoder,
+    FLAC__byte buffer[], size_t * bytes, void *client_data);
+static FLAC__StreamDecoderReadStatus
+gst_flac_dec_read_stream (const FLAC__StreamDecoder * decoder,
+    FLAC__byte buffer[], size_t * bytes, void *client_data);
+static FLAC__StreamDecoderSeekStatus
+gst_flac_dec_seek (const FLAC__StreamDecoder * decoder,
+    FLAC__uint64 position, void *client_data);
+static FLAC__StreamDecoderTellStatus
+gst_flac_dec_tell (const FLAC__StreamDecoder * decoder,
+    FLAC__uint64 * position, void *client_data);
+static FLAC__StreamDecoderLengthStatus
+gst_flac_dec_length (const FLAC__StreamDecoder * decoder,
+    FLAC__uint64 * length, void *client_data);
+static FLAC__bool gst_flac_dec_eof (const FLAC__StreamDecoder * decoder,
+    void *client_data);
+static FLAC__StreamDecoderWriteStatus
+gst_flac_dec_write_stream (const FLAC__StreamDecoder * decoder,
+    const FLAC__Frame * frame,
+    const FLAC__int32 * const buffer[], void *client_data);
+static void gst_flac_dec_metadata_callback_stream (const FLAC__StreamDecoder *
+    decoder, const FLAC__StreamMetadata * metadata, void *client_data);
+static void gst_flac_dec_error_callback_stream (const FLAC__StreamDecoder *
+    decoder, FLAC__StreamDecoderErrorStatus status, void *client_data);
+#endif
 
 GST_BOILERPLATE (GstFlacDec, gst_flac_dec, GstElement, GST_TYPE_ELEMENT);
 #define GST_FLAC_DEC_SRC_CAPS                             \
@@ -212,7 +240,11 @@ static void
 gst_flac_dec_reset_decoders (GstFlacDec * flacdec)
 {
   if (flacdec->seekable_decoder) {
+#ifdef LEGACY_FLAC
     FLAC__seekable_stream_decoder_delete (flacdec->seekable_decoder);
+#else
+    FLAC__stream_decoder_delete (flacdec->seekable_decoder);
+#endif
     flacdec->seekable_decoder = NULL;
   }
 
@@ -228,6 +260,19 @@ gst_flac_dec_reset_decoders (GstFlacDec * flacdec)
     flacdec->adapter = NULL;
   }
 
+  if (flacdec->close_segment) {
+    gst_event_unref (flacdec->close_segment);
+    flacdec->close_segment = NULL;
+  }
+  if (flacdec->start_segment) {
+    gst_event_unref (flacdec->start_segment);
+    flacdec->start_segment = NULL;
+  }
+  if (flacdec->tags) {
+    gst_tag_list_free (flacdec->tags);
+    flacdec->tags = NULL;
+  }
+
   flacdec->segment.last_stop = 0;
   flacdec->offset = 0;
   flacdec->init = TRUE;
@@ -238,6 +283,7 @@ gst_flac_dec_setup_seekable_decoder (GstFlacDec * dec)
 {
   gst_flac_dec_reset_decoders (dec);
 
+#ifdef LEGACY_FLAC
   dec->seekable_decoder = FLAC__seekable_stream_decoder_new ();
 
   FLAC__seekable_stream_decoder_set_read_callback (dec->seekable_decoder,
@@ -259,6 +305,14 @@ gst_flac_dec_setup_seekable_decoder (GstFlacDec * dec)
   FLAC__seekable_stream_decoder_set_error_callback (dec->seekable_decoder,
       gst_flac_dec_error_callback_seekable);
   FLAC__seekable_stream_decoder_set_client_data (dec->seekable_decoder, dec);
+#else
+  dec->seekable_decoder = FLAC__stream_decoder_new ();
+  FLAC__stream_decoder_set_metadata_respond (dec->seekable_decoder,
+      FLAC__METADATA_TYPE_VORBIS_COMMENT);
+  FLAC__stream_decoder_set_metadata_respond (dec->seekable_decoder,
+      FLAC__METADATA_TYPE_PICTURE);
+  FLAC__stream_decoder_set_md5_checking (dec->seekable_decoder, false); /* no point calculating since it's never checked here */
+#endif
 }
 
 static void
@@ -270,6 +324,7 @@ gst_flac_dec_setup_stream_decoder (GstFlacDec * dec)
 
   dec->stream_decoder = FLAC__stream_decoder_new ();
 
+#ifdef LEGACY_FLAC
   FLAC__stream_decoder_set_read_callback (dec->stream_decoder,
       gst_flac_dec_read_stream);
   FLAC__stream_decoder_set_write_callback (dec->stream_decoder,
@@ -281,6 +336,13 @@ gst_flac_dec_setup_stream_decoder (GstFlacDec * dec)
   FLAC__stream_decoder_set_error_callback (dec->stream_decoder,
       gst_flac_dec_error_callback_stream);
   FLAC__stream_decoder_set_client_data (dec->stream_decoder, dec);
+#else
+  FLAC__stream_decoder_set_md5_checking (dec->stream_decoder, false);   /* no point calculating since it's never checked here */
+  FLAC__stream_decoder_set_metadata_respond (dec->stream_decoder,
+      FLAC__METADATA_TYPE_VORBIS_COMMENT);
+  FLAC__stream_decoder_set_metadata_respond (dec->stream_decoder,
+      FLAC__METADATA_TYPE_PICTURE);
+#endif
 }
 
 static void
@@ -327,7 +389,10 @@ gst_flac_dec_update_metadata (GstFlacDec * flacdec,
   gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
       GST_TAG_AUDIO_CODEC, "FLAC", NULL);
 
-  gst_element_found_tags_for_pad (GST_ELEMENT (flacdec), flacdec->srcpad, list);
+  if (flacdec->tags)
+    gst_tag_list_free (flacdec->tags);
+  flacdec->tags = list;
+
 
   return TRUE;
 }
@@ -504,6 +569,132 @@ gst_flac_dec_scan_for_last_block (GstFlacDec * flacdec, gint64 * samples)
   }
 }
 
+/* FIXME: remove ifndef once we depend on flac >= 1.2.x */
+#ifndef LEGACY_FLAC
+static gchar *
+gst_flac_normalize_picture_mime_type (const gchar * old_mime_type,
+    gboolean * is_pic_uri)
+{
+  gchar *mime_str;
+
+  g_return_val_if_fail (old_mime_type != NULL, NULL);
+
+  /* Make lower-case */
+  mime_str = g_ascii_strdown (old_mime_type, -1);
+
+  /* Fix up 'jpg' => 'jpeg' in mime/media type */
+  if (g_ascii_strcasecmp (mime_str, "jpg") == 0 ||
+      g_ascii_strcasecmp (mime_str, "image/jpg") == 0) {
+    g_free (mime_str);
+    mime_str = g_strdup ("image/jpeg");
+  }
+
+  /* Check if the picture is a URI reference */
+  *is_pic_uri = (strcmp (mime_str, "-->") == 0);
+
+  if (!(*is_pic_uri) && *mime_str && strchr (mime_str, '/') == NULL) {
+    gchar *tmp = g_strdup_printf ("image/%s", mime_str);
+
+    g_free (mime_str);
+    mime_str = tmp;
+  }
+
+  return mime_str;
+}
+
+static void
+gst_flac_extract_picture_buffer (GstFlacDec * flacdec,
+    const FLAC__StreamMetadata * metadata)
+{
+  /* Most of this is copied from gst/id3demux/id3v2frames.c */
+  gchar *mime_type;
+  GstBuffer *image;
+  GstCaps *image_caps;
+  FLAC__StreamMetadata_Picture picture;
+  gboolean is_pic_uri;
+
+  g_return_if_fail (metadata->type == FLAC__METADATA_TYPE_PICTURE);
+
+  GST_LOG ("Got PICTURE block");
+  picture = metadata->data.picture;
+
+  is_pic_uri = FALSE;
+  mime_type = gst_flac_normalize_picture_mime_type (picture.mime_type,
+      &is_pic_uri);
+
+  GST_DEBUG ("PICTURE MIME-type is: \"%s\"", mime_type);
+  GST_DEBUG ("image data is %u bytes", picture.data_length);
+
+  if (is_pic_uri) {
+    gchar *uri;
+
+    uri = g_strndup ((gchar *) picture.data, picture.data_length);
+    GST_DEBUG ("image URI: %s", uri);
+
+    image = gst_buffer_new ();
+    GST_BUFFER_MALLOCDATA (image) = (guint8 *) uri;     /* take ownership */
+    GST_BUFFER_DATA (image) = (guint8 *) uri;
+    GST_BUFFER_SIZE (image) = picture.data_length;
+
+    image_caps = gst_caps_new_simple ("text/uri-list", NULL);
+  }
+
+  else {
+    image = gst_buffer_new_and_alloc (picture.data_length);
+    memcpy (GST_BUFFER_DATA (image), picture.data, picture.data_length);
+
+    /* if possible use GStreamer media type rather than declared type */
+    image_caps = gst_type_find_helper_for_buffer (NULL, image, NULL);
+    if (image_caps) {
+      GST_DEBUG ("Found GStreamer media type: %" GST_PTR_FORMAT, image_caps);
+    } else if (mime_type && *mime_type) {
+      GST_DEBUG ("No GStreamer media type found, using declared type: \"%s\"",
+          mime_type);
+      image_caps = gst_caps_new_simple (mime_type, NULL);
+    } else {
+      GST_DEBUG ("Empty declared mime type, ignoring image frame");
+      image = NULL;
+      image_caps = NULL;
+    }
+  }
+
+  g_free (mime_type);
+
+  if (image && image_caps) {
+    GstTagList *tags = gst_tag_list_new ();
+
+    if (picture.type == 1 || picture.type == 2) {
+      /* file icon for preview. Don't add image-type to caps, since it only 
+       * makes sense for there to be one of these. */
+      gst_buffer_set_caps (image, image_caps);
+      gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
+          GST_TAG_PREVIEW_IMAGE, image, NULL);
+    } else {
+      GstTagImageType gst_tag_pic_type = GST_TAG_IMAGE_TYPE_UNDEFINED;
+
+      if (picture.type >= 0x03 && picture.type <= 0x14)
+        gst_tag_pic_type = (GstTagImageType) ((gint) (picture.type) - 2);
+
+      gst_structure_set (gst_caps_get_structure (image_caps, 0),
+          "image-type", GST_TYPE_TAG_IMAGE_TYPE, gst_tag_pic_type, NULL);
+
+      gst_buffer_set_caps (image, image_caps);
+      gst_tag_list_add (tags, GST_TAG_MERGE_APPEND, GST_TAG_IMAGE, image, NULL);
+    }
+    gst_caps_unref (image_caps);
+    gst_buffer_unref (image);
+
+    /* Announce discovered tags */
+    gst_element_found_tags_for_pad (GST_ELEMENT (flacdec), flacdec->srcpad,
+        tags);
+  } else {
+    if (image)
+      gst_buffer_unref (image);
+    GST_DEBUG ("problem parsing PICTURE block, skipping");
+  }
+}
+#endif /* LEGACY_FLAC */
+
 static void
 gst_flac_dec_metadata_callback (GstFlacDec * flacdec,
     const FLAC__StreamMetadata * metadata)
@@ -529,13 +720,34 @@ gst_flac_dec_metadata_callback (GstFlacDec * flacdec,
       GST_DEBUG_OBJECT (flacdec, "total samples = %" G_GINT64_FORMAT, samples);
 
       if (samples > 0) {
+        gint64 duration;
+
         gst_segment_set_duration (&flacdec->segment, GST_FORMAT_DEFAULT,
             samples);
-        if (flacdec->segment.stop == -1)
-          flacdec->segment.stop = samples;
+
+        /* convert duration to time */
+        duration = gst_util_uint64_scale_int (samples, GST_SECOND,
+            flacdec->sample_rate);
+
+        /* fixme, at this time we could seek to the queued seek event if we have
+         * any */
+        if (flacdec->start_segment)
+          gst_event_unref (flacdec->start_segment);
+        flacdec->start_segment =
+            gst_event_new_new_segment_full (FALSE,
+            flacdec->segment.rate, flacdec->segment.applied_rate,
+            GST_FORMAT_TIME, 0, duration, 0);
       }
       break;
     }
+
+/* FIXME: remove ifndef once we depend on flac >= 1.2.x */
+#ifndef LEGACY_FLAC
+    case FLAC__METADATA_TYPE_PICTURE:{
+      gst_flac_extract_picture_buffer (flacdec, metadata);
+      break;
+    }
+#endif
     case FLAC__METADATA_TYPE_VORBIS_COMMENT:
       gst_flac_dec_update_metadata (flacdec, metadata);
       break;
@@ -544,6 +756,7 @@ gst_flac_dec_metadata_callback (GstFlacDec * flacdec,
   }
 }
 
+#ifdef LEGACY_FLAC
 static void
 gst_flac_dec_metadata_callback_seekable (const FLAC__SeekableStreamDecoder * d,
     const FLAC__StreamMetadata * metadata, void *client_data)
@@ -552,6 +765,7 @@ gst_flac_dec_metadata_callback_seekable (const FLAC__SeekableStreamDecoder * d,
 
   gst_flac_dec_metadata_callback (dec, metadata);
 }
+#endif
 
 static void
 gst_flac_dec_metadata_callback_stream (const FLAC__StreamDecoder * decoder,
@@ -587,12 +801,14 @@ gst_flac_dec_error_callback (GstFlacDec * dec,
   dec->last_flow = GST_FLOW_ERROR;
 }
 
+#ifdef LEGACY_FLAC
 static void
 gst_flac_dec_error_callback_seekable (const FLAC__SeekableStreamDecoder * d,
     FLAC__StreamDecoderErrorStatus status, void *client_data)
 {
   gst_flac_dec_error_callback (GST_FLAC_DEC (client_data), status);
 }
+#endif
 
 static void
 gst_flac_dec_error_callback_stream (const FLAC__StreamDecoder * d,
@@ -601,9 +817,15 @@ gst_flac_dec_error_callback_stream (const FLAC__StreamDecoder * d,
   gst_flac_dec_error_callback (GST_FLAC_DEC (client_data), status);
 }
 
+#ifdef LEGACY_FLAC
 static FLAC__SeekableStreamDecoderSeekStatus
 gst_flac_dec_seek (const FLAC__SeekableStreamDecoder * decoder,
     FLAC__uint64 position, void *client_data)
+#else
+static FLAC__StreamDecoderSeekStatus
+gst_flac_dec_seek (const FLAC__StreamDecoder * decoder,
+    FLAC__uint64 position, void *client_data)
+#endif
 {
   GstFlacDec *flacdec;
 
@@ -612,12 +834,22 @@ gst_flac_dec_seek (const FLAC__SeekableStreamDecoder * decoder,
   GST_DEBUG ("seek %" G_GINT64_FORMAT, position);
   flacdec->offset = position;
 
+#ifdef LEGACY_FLAC
   return FLAC__SEEKABLE_STREAM_DECODER_SEEK_STATUS_OK;
+#else
+  return FLAC__STREAM_DECODER_SEEK_STATUS_OK;
+#endif
 }
 
+#ifdef LEGACY_FLAC
 static FLAC__SeekableStreamDecoderTellStatus
 gst_flac_dec_tell (const FLAC__SeekableStreamDecoder * decoder,
     FLAC__uint64 * position, void *client_data)
+#else
+static FLAC__StreamDecoderTellStatus
+gst_flac_dec_tell (const FLAC__StreamDecoder * decoder,
+    FLAC__uint64 * position, void *client_data)
+#endif
 {
   GstFlacDec *flacdec;
 
@@ -627,12 +859,22 @@ gst_flac_dec_tell (const FLAC__SeekableStreamDecoder * decoder,
 
   GST_DEBUG ("tell %" G_GINT64_FORMAT, *position);
 
+#ifdef LEGACY_FLAC
   return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_OK;
+#else
+  return FLAC__STREAM_DECODER_TELL_STATUS_OK;
+#endif
 }
 
+#ifdef LEGACY_FLAC
 static FLAC__SeekableStreamDecoderLengthStatus
 gst_flac_dec_length (const FLAC__SeekableStreamDecoder * decoder,
     FLAC__uint64 * length, void *client_data)
+#else
+static FLAC__StreamDecoderLengthStatus
+gst_flac_dec_length (const FLAC__StreamDecoder * decoder,
+    FLAC__uint64 * length, void *client_data)
+#endif
 {
   GstFlacDec *flacdec;
   GstFormat fmt = GST_FORMAT_BYTES;
@@ -642,22 +884,39 @@ gst_flac_dec_length (const FLAC__SeekableStreamDecoder * decoder,
   flacdec = GST_FLAC_DEC (client_data);
 
   if (!(peer = gst_pad_get_peer (flacdec->sinkpad)))
+#ifdef LEGACY_FLAC
     return FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_ERROR;
+#else
+    return FLAC__STREAM_DECODER_LENGTH_STATUS_ERROR;
+#endif
   gst_pad_query_duration (peer, &fmt, &len);
   gst_object_unref (peer);
   if (fmt != GST_FORMAT_BYTES || len == -1)
+#ifdef LEGACY_FLAC
     return FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_ERROR;
+#else
+    return FLAC__STREAM_DECODER_LENGTH_STATUS_ERROR;
+#endif
 
   *length = len;
 
   GST_DEBUG ("length %" G_GINT64_FORMAT, *length);
 
+#ifdef LEGACY_FLAC
   return FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_OK;
+#else
+  return FLAC__STREAM_DECODER_LENGTH_STATUS_OK;
+#endif
 }
 
+#ifdef LEGACY_FLAC
 static FLAC__bool
 gst_flac_dec_eof (const FLAC__SeekableStreamDecoder * decoder,
     void *client_data)
+#else
+static FLAC__bool
+gst_flac_dec_eof (const FLAC__StreamDecoder * decoder, void *client_data)
+#endif
 {
   GstFlacDec *flacdec;
   GstFormat fmt;
@@ -685,9 +944,15 @@ gst_flac_dec_eof (const FLAC__SeekableStreamDecoder * decoder,
   return ret;
 }
 
+#ifdef LEGACY_FLAC
 static FLAC__SeekableStreamDecoderReadStatus
 gst_flac_dec_read_seekable (const FLAC__SeekableStreamDecoder * decoder,
     FLAC__byte buffer[], unsigned *bytes, void *client_data)
+#else
+static FLAC__StreamDecoderReadStatus
+gst_flac_dec_read_seekable (const FLAC__StreamDecoder * decoder,
+    FLAC__byte buffer[], size_t * bytes, void *client_data)
+#endif
 {
   GstFlacDec *flacdec;
   GstBuffer *buf;
@@ -696,7 +961,11 @@ gst_flac_dec_read_seekable (const FLAC__SeekableStreamDecoder * decoder,
 
   if (gst_pad_pull_range (flacdec->sinkpad, flacdec->offset, *bytes,
           &buf) != GST_FLOW_OK)
+#ifdef LEGACY_FLAC
     return FLAC__SEEKABLE_STREAM_DECODER_READ_STATUS_ERROR;
+#else
+    return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
+#endif
 
   GST_DEBUG ("Read %d bytes at %" G_GUINT64_FORMAT,
       GST_BUFFER_SIZE (buf), flacdec->offset);
@@ -705,12 +974,22 @@ gst_flac_dec_read_seekable (const FLAC__SeekableStreamDecoder * decoder,
   gst_buffer_unref (buf);
   flacdec->offset += *bytes;
 
+#ifdef LEGACY_FLAC
   return FLAC__SEEKABLE_STREAM_DECODER_READ_STATUS_OK;
+#else
+  return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
+#endif
 }
 
+#ifdef LEGACY_FLAC
 static FLAC__StreamDecoderReadStatus
 gst_flac_dec_read_stream (const FLAC__StreamDecoder * decoder,
     FLAC__byte buffer[], unsigned *bytes, void *client_data)
+#else
+static FLAC__StreamDecoderReadStatus
+gst_flac_dec_read_stream (const FLAC__StreamDecoder * decoder,
+    FLAC__byte buffer[], size_t * bytes, void *client_data)
+#endif
 {
   GstFlacDec *dec = GST_FLAC_DEC (client_data);
   guint len;
@@ -743,6 +1022,7 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
   guint channels = frame->header.channels;
   guint samples = frame->header.blocksize;
   guint j, i;
+  GstClockTime next;
 
   switch (depth) {
     case 8:
@@ -758,7 +1038,9 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
       width = 32;
       break;
     default:
-      g_assert_not_reached ();
+      GST_ERROR_OBJECT (flacdec, "unsupported depth %d", depth);
+      ret = GST_FLOW_ERROR;
+      goto done;
   }
 
   if (!GST_PAD_CAPS (flacdec->srcpad)) {
@@ -775,25 +1057,28 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
         "rate", G_TYPE_INT, frame->header.sample_rate,
         "channels", G_TYPE_INT, channels, NULL);
 
-    if (!gst_pad_set_caps (flacdec->srcpad, caps)) {
-      GST_ELEMENT_ERROR (flacdec, CORE, NEGOTIATION, (NULL),
-          ("Failed to negotiate caps %" GST_PTR_FORMAT, caps));
-      flacdec->last_flow = GST_FLOW_ERROR;
-      gst_caps_unref (caps);
-      return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
-    }
-
-    gst_caps_unref (caps);
-
     flacdec->depth = depth;
     flacdec->width = width;
     flacdec->channels = channels;
     flacdec->sample_rate = frame->header.sample_rate;
+
+    gst_pad_set_caps (flacdec->srcpad, caps);
+    gst_caps_unref (caps);
   }
 
-  if (flacdec->need_newsegment) {
-    gst_flac_dec_send_newsegment (flacdec, FALSE);
-    flacdec->need_newsegment = FALSE;
+  if (flacdec->close_segment) {
+    gst_pad_push_event (flacdec->srcpad, flacdec->close_segment);
+    flacdec->close_segment = NULL;
+  }
+  if (flacdec->start_segment) {
+    gst_pad_push_event (flacdec->srcpad, flacdec->start_segment);
+    flacdec->start_segment = NULL;
+  }
+
+  if (flacdec->tags) {
+    gst_element_found_tags_for_pad (GST_ELEMENT (flacdec), flacdec->srcpad,
+        flacdec->tags);
+    flacdec->tags = NULL;
   }
 
   ret = gst_pad_alloc_buffer_and_set_caps (flacdec->srcpad,
@@ -816,9 +1101,11 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
       gst_util_uint64_scale_int (flacdec->segment.last_stop, GST_SECOND,
       frame->header.sample_rate);
 
-  GST_BUFFER_DURATION (outbuf) =
-      gst_util_uint64_scale_int (samples, GST_SECOND,
-      frame->header.sample_rate);
+  /* get next timestamp to calculate the duration */
+  next = gst_util_uint64_scale_int (flacdec->segment.last_stop + samples,
+      GST_SECOND, frame->header.sample_rate);
+
+  GST_BUFFER_DURATION (outbuf) = next - GST_BUFFER_TIMESTAMP (outbuf);
 
   if (depth == 8) {
     gint8 *outbuffer = (gint8 *) GST_BUFFER_DATA (outbuf);
@@ -855,6 +1142,11 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
         GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
         GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)));
 
+    if (flacdec->discont) {
+      outbuf = gst_buffer_make_metadata_writable (outbuf);
+      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
+      flacdec->discont = FALSE;
+    }
     ret = gst_pad_push (flacdec->srcpad, outbuf);
   } else {
     GST_DEBUG ("not pushing %d samples at offset %" G_GINT64_FORMAT
@@ -880,6 +1172,7 @@ done:
   return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
 }
 
+#ifdef LEGACY_FLAC
 static FLAC__StreamDecoderWriteStatus
 gst_flac_dec_write_seekable (const FLAC__SeekableStreamDecoder * decoder,
     const FLAC__Frame * frame,
@@ -887,6 +1180,7 @@ gst_flac_dec_write_seekable (const FLAC__SeekableStreamDecoder * decoder,
 {
   return gst_flac_dec_write (GST_FLAC_DEC (client_data), frame, buffer);
 }
+#endif
 
 static FLAC__StreamDecoderWriteStatus
 gst_flac_dec_write_stream (const FLAC__StreamDecoder * decoder,
@@ -900,7 +1194,13 @@ static void
 gst_flac_dec_loop (GstPad * sinkpad)
 {
   GstFlacDec *flacdec;
+
+#ifdef LEGACY_FLAC
   FLAC__SeekableStreamDecoderState s;
+#else
+  FLAC__StreamDecoderState s;
+  FLAC__StreamDecoderInitStatus is;
+#endif
 
   flacdec = GST_FLAC_DEC (GST_OBJECT_PARENT (sinkpad));
 
@@ -908,9 +1208,19 @@ gst_flac_dec_loop (GstPad * sinkpad)
 
   if (flacdec->init) {
     GST_DEBUG_OBJECT (flacdec, "initializing decoder");
+#ifdef LEGACY_FLAC
     s = FLAC__seekable_stream_decoder_init (flacdec->seekable_decoder);
     if (s != FLAC__SEEKABLE_STREAM_DECODER_OK)
       goto analyze_state;
+#else
+    is = FLAC__stream_decoder_init_stream (flacdec->seekable_decoder,
+        gst_flac_dec_read_seekable, gst_flac_dec_seek, gst_flac_dec_tell,
+        gst_flac_dec_length, gst_flac_dec_eof, gst_flac_dec_write_stream,
+        gst_flac_dec_metadata_callback_stream,
+        gst_flac_dec_error_callback_stream, flacdec);
+    if (is != FLAC__STREAM_DECODER_INIT_STATUS_OK)
+      goto analyze_state;
+#endif
     /*    FLAC__seekable_stream_decoder_process_metadata (flacdec->seekable_decoder); */
     flacdec->init = FALSE;
   }
@@ -920,21 +1230,40 @@ gst_flac_dec_loop (GstPad * sinkpad)
   flacdec->last_flow = GST_FLOW_OK;
 
   GST_LOG_OBJECT (flacdec, "processing single");
+#ifdef LEGACY_FLAC
   FLAC__seekable_stream_decoder_process_single (flacdec->seekable_decoder);
-
+#else
+  FLAC__stream_decoder_process_single (flacdec->seekable_decoder);
+#endif
 analyze_state:
 
   GST_LOG_OBJECT (flacdec, "done processing, checking encoder state");
+#ifdef LEGACY_FLAC
   s = FLAC__seekable_stream_decoder_get_state (flacdec->seekable_decoder);
+#else
+  s = FLAC__stream_decoder_get_state (flacdec->seekable_decoder);
+#endif
   switch (s) {
+#ifdef LEGACY_FLAC
     case FLAC__SEEKABLE_STREAM_DECODER_OK:
-    case FLAC__SEEKABLE_STREAM_DECODER_SEEKING:{
+    case FLAC__SEEKABLE_STREAM_DECODER_SEEKING:
+#else
+    case FLAC__STREAM_DECODER_SEARCH_FOR_METADATA:
+    case FLAC__STREAM_DECODER_READ_METADATA:
+    case FLAC__STREAM_DECODER_SEARCH_FOR_FRAME_SYNC:
+    case FLAC__STREAM_DECODER_READ_FRAME:
+#endif
+    {
       GST_DEBUG_OBJECT (flacdec, "everything ok");
 
-      if (flacdec->last_flow != GST_FLOW_OK &&
-          flacdec->last_flow != GST_FLOW_NOT_LINKED) {
-        GST_DEBUG_OBJECT (flacdec, "last_flow return was %s, pausing",
-            gst_flow_get_name (flacdec->last_flow));
+      if (GST_FLOW_IS_FATAL (flacdec->last_flow) ||
+          flacdec->last_flow == GST_FLOW_NOT_LINKED) {
+        GST_ELEMENT_ERROR (flacdec, STREAM, FAILED,
+            (_("Internal data stream error.")),
+            ("stream stopped, reason %s",
+                gst_flow_get_name (flacdec->last_flow)));
+        goto eos_and_pause;
+      } else if (flacdec->last_flow != GST_FLOW_OK) {
         goto pause;
       }
 
@@ -956,9 +1285,17 @@ analyze_state:
       return;
     }
 
+#ifdef LEGACY_FLAC
     case FLAC__SEEKABLE_STREAM_DECODER_END_OF_STREAM:{
+#else
+    case FLAC__STREAM_DECODER_END_OF_STREAM:{
+#endif
       GST_DEBUG_OBJECT (flacdec, "EOS");
+#ifdef LEGACY_FLAC
       FLAC__seekable_stream_decoder_reset (flacdec->seekable_decoder);
+#else
+      FLAC__stream_decoder_reset (flacdec->seekable_decoder);
+#endif
 
       if ((flacdec->segment.flags & GST_SEEK_FLAG_SEGMENT) != 0) {
         if (flacdec->segment.duration > 0) {
@@ -972,6 +1309,7 @@ analyze_state:
       goto eos_and_pause;
     }
 
+#ifdef LEGACY_FLAC
     case FLAC__SEEKABLE_STREAM_DECODER_MEMORY_ALLOCATION_ERROR:
     case FLAC__SEEKABLE_STREAM_DECODER_STREAM_DECODER_ERROR:
     case FLAC__SEEKABLE_STREAM_DECODER_READ_ERROR:
@@ -979,11 +1317,23 @@ analyze_state:
     case FLAC__SEEKABLE_STREAM_DECODER_ALREADY_INITIALIZED:
     case FLAC__SEEKABLE_STREAM_DECODER_INVALID_CALLBACK:
     case FLAC__SEEKABLE_STREAM_DECODER_UNINITIALIZED:
+#else
+    case FLAC__STREAM_DECODER_OGG_ERROR:
+    case FLAC__STREAM_DECODER_SEEK_ERROR:
+    case FLAC__STREAM_DECODER_ABORTED:
+    case FLAC__STREAM_DECODER_MEMORY_ALLOCATION_ERROR:
+    case FLAC__STREAM_DECODER_UNINITIALIZED:
+#endif
     default:{
       /* fixme: this error sucks -- should try to figure out when/if an more
          specific error was already sent via the callback */
+#ifdef LEGACY_FLAC
       GST_ELEMENT_ERROR (flacdec, STREAM, DECODE, (NULL),
           ("%s", FLAC__SeekableStreamDecoderStateString[s]));
+#else
+      GST_ELEMENT_ERROR (flacdec, STREAM, DECODE, (NULL),
+          ("%s", FLAC__StreamDecoderStateString[s]));
+#endif
       goto eos_and_pause;
     }
   }
@@ -1006,14 +1356,13 @@ segment_done_and_pause:
 
     goto pause;
   }
-
 eos_and_pause:
   {
     GST_DEBUG_OBJECT (flacdec, "sending EOS event");
+    flacdec->running = FALSE;
     gst_pad_push_event (flacdec->srcpad, gst_event_new_eos ());
     /* fall through to pause */
   }
-
 pause:
   {
     GST_DEBUG_OBJECT (flacdec, "pausing");
@@ -1054,7 +1403,6 @@ gst_flac_dec_sink_event (GstPad * pad, GstEvent * event)
         GST_DEBUG_OBJECT (dec, "newsegment event in TIME format => framed");
         dec->framed = TRUE;
         res = gst_pad_push_event (dec->srcpad, event);
-        dec->need_newsegment = FALSE;
 
         /* this won't work for the first newsegment event though ... */
         if (gst_flac_dec_convert_src (dec->srcpad, GST_FORMAT_TIME, cur,
@@ -1071,7 +1419,6 @@ gst_flac_dec_sink_event (GstPad * pad, GstEvent * event)
         GST_DEBUG_OBJECT (dec, "newsegment event in %s format => not framed",
             gst_format_get_name (fmt));
         dec->framed = FALSE;
-        dec->need_newsegment = TRUE;
         gst_event_unref (event);
         res = TRUE;
       }
@@ -1101,7 +1448,11 @@ gst_flac_dec_sink_event (GstPad * pad, GstEvent * event)
 static GstFlowReturn
 gst_flac_dec_chain (GstPad * pad, GstBuffer * buf)
 {
+#ifdef  LEGACY_FLAC
   FLAC__StreamDecoderState s;
+#else
+  FLAC__StreamDecoderInitStatus s;
+#endif
   GstFlacDec *dec;
   gboolean got_audio_frame;
 
@@ -1113,11 +1464,22 @@ gst_flac_dec_chain (GstPad * pad, GstBuffer * buf)
 
   if (dec->init) {
     GST_DEBUG_OBJECT (dec, "initializing decoder");
+#ifdef  LEGACY_FLAC
     s = FLAC__stream_decoder_init (dec->stream_decoder);
     if (s != FLAC__STREAM_DECODER_SEARCH_FOR_METADATA) {
       GST_ELEMENT_ERROR (GST_ELEMENT (dec), LIBRARY, INIT, (NULL), (NULL));
       return GST_FLOW_ERROR;
     }
+#else
+    s = FLAC__stream_decoder_init_stream (dec->stream_decoder,
+        gst_flac_dec_read_stream, NULL, NULL, NULL, NULL,
+        gst_flac_dec_write_stream, gst_flac_dec_metadata_callback_stream,
+        gst_flac_dec_error_callback_stream, dec);
+    if (s != FLAC__STREAM_DECODER_INIT_STATUS_OK) {
+      GST_ELEMENT_ERROR (GST_ELEMENT (dec), LIBRARY, INIT, (NULL), (NULL));
+      return GST_FLOW_ERROR;
+    }
+#endif
     GST_DEBUG_OBJECT (dec, "initialized (framed=%d)", dec->framed);
     dec->init = FALSE;
   }
@@ -1481,37 +1843,6 @@ done:
   return res;
 }
 
-static void
-gst_flac_dec_send_newsegment (GstFlacDec * flacdec, gboolean update)
-{
-  GstSegment *s = &flacdec->segment;
-  GstFormat target_format = GST_FORMAT_TIME;
-  gint64 stop_time = -1;
-  gint64 start_time = 0;
-
-  /* segment is in DEFAULT format, but we want to send a TIME newsegment */
-  if (!gst_flac_dec_convert_src (flacdec->srcpad, GST_FORMAT_DEFAULT,
-          s->start, &target_format, &start_time)) {
-    GST_WARNING_OBJECT (flacdec, "failed to convert segment start %lld to TIME",
-        s->start);
-    return;
-  }
-
-  if (s->stop != -1 && !gst_flac_dec_convert_src (flacdec->srcpad,
-          GST_FORMAT_DEFAULT, s->stop, &target_format, &stop_time)) {
-    GST_WARNING_OBJECT (flacdec, "failed to convert segment stop to TIME");
-    return;
-  }
-
-  GST_DEBUG_OBJECT (flacdec, "sending newsegment from %" GST_TIME_FORMAT
-      " to %" GST_TIME_FORMAT, GST_TIME_ARGS (start_time),
-      GST_TIME_ARGS (stop_time));
-
-  gst_pad_push_event (flacdec->srcpad,
-      gst_event_new_new_segment (update, s->rate, GST_FORMAT_TIME,
-          start_time, stop_time, start_time));
-}
-
 static gboolean
 gst_flac_dec_handle_seek_event (GstFlacDec * flacdec, GstEvent * event)
 {
@@ -1524,7 +1855,7 @@ gst_flac_dec_handle_seek_event (GstFlacDec * flacdec, GstEvent * event)
   gboolean only_update = FALSE;
   gboolean flush;
   gdouble rate;
-  gint64 start;
+  gint64 start, last_stop;
   gint64 stop;
 
   if (flacdec->seekable_decoder == NULL) {
@@ -1566,23 +1897,29 @@ gst_flac_dec_handle_seek_event (GstFlacDec * flacdec, GstEvent * event)
   flush = ((seek_flags & GST_SEEK_FLAG_FLUSH) == GST_SEEK_FLAG_FLUSH);
 
   if (flush) {
+    /* flushing seek, clear the pipeline of stuff, we need a newsegment after
+     * this. */
+    GST_DEBUG_OBJECT (flacdec, "flushing");
     gst_pad_push_event (flacdec->srcpad, gst_event_new_flush_start ());
   } else {
+    /* non flushing seek, pause the task */
+    GST_DEBUG_OBJECT (flacdec, "stopping task");
     gst_pad_stop_task (flacdec->sinkpad);
   }
 
+  /* acquire the stream lock, this either happens when the streaming thread
+   * stopped because of the flush or when the task is paused after the loop
+   * function finished an iteration, which can never happen when it's blocked
+   * downstream in PAUSED, for example */
   GST_PAD_STREAM_LOCK (flacdec->sinkpad);
 
-  /* operate on segment copy until we know the seek worked */
+  /* operate on segment copy until we know the seek worked. The idea is that
+   * when the seek fails, we want to continue with what we were doing without
+   * having the segment being updated. */
   segment = flacdec->segment;
 
-  if (segment.duration > 0) {
-    if (start != -1)
-      start = CLAMP (start, 0, segment.duration - 1);
-    if (stop != -1)
-      stop = CLAMP (stop, 0, segment.duration - 1);
-  }
-
+  /* update the segment with the seek values, last_stop will contain the new
+   * position we should seek to */
   gst_segment_set_seek (&segment, rate, GST_FORMAT_DEFAULT,
       seek_flags, start_type, start, stop_type, stop, &only_update);
 
@@ -1593,49 +1930,113 @@ gst_flac_dec_handle_seek_event (GstFlacDec * flacdec, GstEvent * event)
       GST_TIME_ARGS (segment.stop * GST_SECOND / flacdec->sample_rate));
 
   GST_DEBUG_OBJECT (flacdec, "performing seek to sample %" G_GINT64_FORMAT,
-      segment.start);
+      segment.last_stop);
 
+  /* mark ourselves as seeking because the above lines will trigger some
+   * callbacks that need to behave differently when seeking */
   flacdec->seeking = TRUE;
 
   seek_ok =
+#ifdef  LEGACY_FLAC
       FLAC__seekable_stream_decoder_seek_absolute (flacdec->seekable_decoder,
-      segment.start);
+      segment.last_stop);
+#else
+      FLAC__stream_decoder_seek_absolute (flacdec->seekable_decoder,
+      segment.last_stop);
+#endif
 
   flacdec->seeking = FALSE;
 
-  if (flush) {
-    gst_pad_push_event (flacdec->srcpad, gst_event_new_flush_stop ());
+  if (!seek_ok) {
+    /* seek failed, segment was not updated and we start streaming again with
+     * the previous segment values */
+    GST_WARNING_OBJECT (flacdec, "seek failed");
+  } else if (!flush && flacdec->running) {
+    /* we are running the current segment and doing a non-flushing seek, 
+     * close the segment first based on the last_stop. */
+    GST_DEBUG_OBJECT (flacdec, "closing running segment %" G_GINT64_FORMAT
+        " to %" G_GINT64_FORMAT, segment.start, segment.last_stop);
+
+    /* convert the old segment values to time to close the old segment */
+    start = gst_util_uint64_scale_int (flacdec->segment.start, GST_SECOND,
+        flacdec->sample_rate);
+    last_stop =
+        gst_util_uint64_scale_int (flacdec->segment.last_stop, GST_SECOND,
+        flacdec->sample_rate);
+
+    /* queue the segment for sending in the stream thread, start and time are
+     * always the same. */
+    if (flacdec->close_segment)
+      gst_event_unref (flacdec->close_segment);
+    flacdec->close_segment =
+        gst_event_new_new_segment_full (TRUE,
+        flacdec->segment.rate, flacdec->segment.applied_rate, GST_FORMAT_TIME,
+        start, last_stop, start);
   }
 
   if (seek_ok) {
+    /* seek succeeded, copy new segment values */
     flacdec->segment = segment;
-    gst_flac_dec_send_newsegment (flacdec, FALSE);
-    flacdec->segment.last_stop = segment.start;
-
     GST_DEBUG_OBJECT (flacdec, "seek successful");
-
-    /* notify start of new segment */
-    if ((flacdec->segment.flags & GST_SEEK_FLAG_SEGMENT) != 0) {
-      GstMessage *msg;
-      gint64 start_time = 0;
-
-      start_time = gst_util_uint64_scale_int (segment.start, GST_SECOND,
-          flacdec->sample_rate);
-
-      msg = gst_message_new_segment_start (GST_OBJECT (flacdec),
-          GST_FORMAT_TIME, start_time);
-      gst_element_post_message (GST_ELEMENT (flacdec), msg);
-    }
-  } else {
-    GST_WARNING_OBJECT (flacdec, "seek failed");
   }
 
+  /* convert the (new) segment values to time, we will need them to generate the
+   * new segment events. */
+  start = gst_util_uint64_scale_int (flacdec->segment.start, GST_SECOND,
+      flacdec->sample_rate);
+  last_stop = gst_util_uint64_scale_int (flacdec->segment.last_stop, GST_SECOND,
+      flacdec->sample_rate);
+
+  /* for deriving a stop position for the playback segment from the seek
+   * segment, we must take the duration when the stop is not set */
+  if (flacdec->segment.stop != -1)
+    stop = gst_util_uint64_scale_int (flacdec->segment.stop, GST_SECOND,
+        flacdec->sample_rate);
+  else
+    stop = gst_util_uint64_scale_int (flacdec->segment.duration, GST_SECOND,
+        flacdec->sample_rate);
+
+  /* notify start of new segment when we were asked to do so. */
+  if (segment.flags & GST_SEEK_FLAG_SEGMENT) {
+    /* last_stop contains the position we start from */
+    gst_element_post_message (GST_ELEMENT (flacdec),
+        gst_message_new_segment_start (GST_OBJECT (flacdec),
+            GST_FORMAT_TIME, last_stop));
+  }
+
+  /* if the seek was ok or (when it failed) we are flushing, we need to send out
+   * a new segment. If we did not flush and the seek failed, we simply do
+   * nothing here and continue where we were. */
+  if (seek_ok || flush) {
+    GST_DEBUG_OBJECT (flacdec, "Creating newsegment from %" G_GINT64_FORMAT
+        " to %" G_GINT64_FORMAT, segment.last_stop, stop);
+    /* now replace the old segment so that we send it in the stream thread the
+     * next time it is scheduled. */
+    if (flacdec->start_segment)
+      gst_event_unref (flacdec->start_segment);
+    flacdec->start_segment =
+        gst_event_new_new_segment_full (FALSE,
+        segment.rate, segment.applied_rate, GST_FORMAT_TIME,
+        last_stop, stop, last_stop);
+  }
+
+  /* prepare for streaming again, start with a flush stop when we issued a flush
+   * start before. */
+  if (flush) {
+    GST_DEBUG_OBJECT (flacdec, "flushing stop");
+    gst_pad_push_event (flacdec->srcpad, gst_event_new_flush_stop ());
+  }
+
+  /* we'll generate a discont on the next buffer */
+  flacdec->discont = TRUE;
+  /* the task is running again now */
+  flacdec->running = TRUE;
   gst_pad_start_task (flacdec->sinkpad,
       (GstTaskFunction) gst_flac_dec_loop, flacdec->sinkpad);
 
   GST_PAD_STREAM_UNLOCK (flacdec->sinkpad);
 
-  return TRUE;
+  return seek_ok;
 }
 
 static gboolean
@@ -1688,6 +2089,8 @@ gst_flac_dec_sink_activate_push (GstPad * sinkpad, gboolean active)
 static gboolean
 gst_flac_dec_sink_activate_pull (GstPad * sinkpad, gboolean active)
 {
+  gboolean res;
+
   if (active) {
     GstFlacDec *flacdec;
 
@@ -1695,11 +2098,14 @@ gst_flac_dec_sink_activate_pull (GstPad * sinkpad, gboolean active)
 
     flacdec->offset = 0;
     gst_flac_dec_setup_seekable_decoder (flacdec);
-    return gst_pad_start_task (sinkpad, (GstTaskFunction) gst_flac_dec_loop,
+    flacdec->running = TRUE;
+
+    res = gst_pad_start_task (sinkpad, (GstTaskFunction) gst_flac_dec_loop,
         sinkpad);
   } else {
-    return gst_pad_stop_task (sinkpad);
+    res = gst_pad_stop_task (sinkpad);
   }
+  return res;
 }
 
 static GstStateChangeReturn
@@ -1710,8 +2116,6 @@ gst_flac_dec_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      flacdec->segment.last_stop = 0;
-      flacdec->need_newsegment = TRUE;
       flacdec->seeking = FALSE;
       flacdec->channels = 0;
       flacdec->depth = 0;
