@@ -13,12 +13,20 @@
  *
  */
 
+/**
+ * SECTION:element-pngdec
+ *
+ * Decodes png images. If there is no framerate set on sink caps, it sends EOS
+ * after the first picture.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include "gstpngdec.h"
 
+#include <stdlib.h>
 #include <string.h>
 #include <gst/video/video.h>
 #include <gst/gst-i18n-plugin.h>
@@ -160,6 +168,8 @@ gst_pngdec_init (GstPngDec * pngdec)
   pngdec->in_duration = GST_CLOCK_TIME_NONE;
 
   gst_segment_init (&pngdec->segment, GST_FORMAT_UNDEFINED);
+
+  pngdec->image_ready = FALSE;
 }
 
 static void
@@ -221,8 +231,8 @@ user_endrow_callback (png_structp png_ptr, png_bytep new_row,
   if (GST_IS_BUFFER (pngdec->buffer_out)) {
     size_t offset = row_num * GST_ROUND_UP_4 (pngdec->rowbytes);
 
-    GST_LOG ("got row %d, copying in buffer %p at offset %d", row_num,
-        pngdec->buffer_out, offset);
+    GST_LOG ("got row %u, copying in buffer %p at offset %" G_GSIZE_FORMAT,
+        (guint) row_num, pngdec->buffer_out, offset);
     memcpy (GST_BUFFER_DATA (pngdec->buffer_out) + offset, new_row,
         pngdec->rowbytes);
     pngdec->ret = GST_FLOW_OK;
@@ -284,18 +294,7 @@ user_end_callback (png_structp png_ptr, png_infop info)
     gst_buffer_unref (pngdec->buffer_out);
   }
   pngdec->buffer_out = NULL;
-
-  if (pngdec->framed) {
-    /* Reset ourselves for the next frame */
-    gst_pngdec_libpng_clear (pngdec);
-    gst_pngdec_libpng_init (pngdec);
-    GST_LOG_OBJECT (pngdec, "setting up callbacks for next frame");
-    png_set_progressive_read_fn (pngdec->png, pngdec,
-        user_info_callback, user_endrow_callback, user_end_callback);
-  } else {
-    GST_LOG_OBJECT (pngdec, "sending EOS");
-    pngdec->ret = gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
-  }
+  pngdec->image_ready = TRUE;
 }
 
 static void
@@ -307,7 +306,8 @@ user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
 
   pngdec = GST_PNGDEC (png_ptr->io_ptr);
 
-  GST_LOG ("reading %d bytes of data at offset %d", length, pngdec->offset);
+  GST_LOG ("reading %" G_GSIZE_FORMAT " bytes of data at offset %d", length,
+      pngdec->offset);
 
   ret = gst_pad_pull_range (pngdec->sinkpad, pngdec->offset, length, &buffer);
   if ((ret != GST_FLOW_OK) || (GST_BUFFER_SIZE (buffer) != length))
@@ -322,9 +322,9 @@ user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
   return;
 
 pause:
-  GST_LOG_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
+  GST_INFO_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
   gst_pad_pause_task (pngdec->sinkpad);
-  if (GST_FLOW_IS_FATAL (ret)) {
+  if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
     gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
     GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
         (_("Internal data stream error.")),
@@ -356,9 +356,14 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
   /* Get Color type */
   color_type = png_get_color_type (pngdec->png, pngdec->info);
 
-  /* HACK: The doc states that it's RGBA but apparently it's not... */
+#if 0
+  /* We used to have this HACK to reverse the outgoing bytes, but the problem
+   * that originally required the hack seems to have been in ffmpegcolorspace's
+   * RGBA descriptions. It doesn't seem needed now that's fixed, but might
+   * still be needed on big-endian systems, I'm not sure. J.S. 6/7/2007 */
   if (color_type == PNG_COLOR_TYPE_RGB_ALPHA)
     png_set_bgr (pngdec->png);
+#endif
 
   /* Gray scale converted to RGB and upscaled to 8 bits */
   if ((color_type == PNG_COLOR_TYPE_GRAY_ALPHA) ||
@@ -451,6 +456,8 @@ gst_pngdec_task (GstPad * pad)
 
   pngdec = GST_PNGDEC (GST_OBJECT_PARENT (pad));
 
+  GST_LOG_OBJECT (pngdec, "read frame");
+
   /* Let libpng come back here on error */
   if (setjmp (png_jmpbuf (pngdec->png))) {
     ret = GST_FLOW_ERROR;
@@ -504,15 +511,13 @@ gst_pngdec_task (GstPad * pad)
   return;
 
 pause:
-  {
-    GST_LOG_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
-    gst_pad_pause_task (pngdec->sinkpad);
-    if (GST_FLOW_IS_FATAL (ret)) {
-      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
-      GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
-          (_("Internal data stream error.")),
-          ("stream stopped, reason %s", gst_flow_get_name (ret)));
-    }
+  GST_INFO_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
+  gst_pad_pause_task (pngdec->sinkpad);
+  if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
+    GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
+        (_("Internal data stream error.")),
+        ("stream stopped, reason %s", gst_flow_get_name (ret)));
+    gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
   }
 }
 
@@ -550,6 +555,21 @@ gst_pngdec_chain (GstPad * pad, GstBuffer * buffer)
   png_process_data (pngdec->png, pngdec->info, GST_BUFFER_DATA (buffer),
       GST_BUFFER_SIZE (buffer));
 
+  if (pngdec->image_ready) {
+    if (pngdec->framed) {
+      /* Reset ourselves for the next frame */
+      gst_pngdec_libpng_clear (pngdec);
+      gst_pngdec_libpng_init (pngdec);
+      GST_LOG_OBJECT (pngdec, "setting up callbacks for next frame");
+      png_set_progressive_read_fn (pngdec->png, pngdec,
+          user_info_callback, user_endrow_callback, user_end_callback);
+    } else {
+      GST_LOG_OBJECT (pngdec, "sending EOS");
+      pngdec->ret = gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+    }
+    pngdec->image_ready = FALSE;
+  }
+
   /* grab new return code */
   ret = pngdec->ret;
 
@@ -586,6 +606,7 @@ gst_pngdec_sink_setcaps (GstPad * pad, GstCaps * caps)
     pngdec->fps_n = num;
     pngdec->fps_d = denom;
   } else {
+    GST_DEBUG_OBJECT (pngdec, "single picture input");
     pngdec->framed = FALSE;
     pngdec->fps_n = 0;
     pngdec->fps_d = 1;
