@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*- */
 /* GStreamer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
  *               <2000> Daniel Fischer <dan@f3c.com>
@@ -42,6 +41,7 @@
 #include <gst/gst.h>
 
 #include "gstdv1394src.h"
+#include "gst1394probe.h"
 
 
 #define CONTROL_STOP            'S'     /* stop the select call */
@@ -94,7 +94,8 @@ enum
   PROP_SKIP,
   PROP_DROP_INCOMPLETE,
   PROP_USE_AVC,
-  PROP_GUID
+  PROP_GUID,
+  PROP_DEVICE_NAME
 };
 
 static const GstElementDetails gst_dv1394src_details =
@@ -133,6 +134,7 @@ static gboolean gst_dv1394src_convert (GstPad * pad,
 
 static const GstQueryType *gst_dv1394src_get_query_types (GstPad * pad);
 static gboolean gst_dv1394src_query (GstPad * pad, GstQuery * query);
+static void gst_dv1394src_update_device_name (GstDV1394Src * src);
 
 static void
 _do_init (GType type)
@@ -144,6 +146,8 @@ _do_init (GType type)
   };
   g_type_add_interface_static (type, GST_TYPE_URI_HANDLER, &urihandler_info);
 
+  gst_1394_type_add_property_probe_interface (type);
+
   GST_DEBUG_CATEGORY_INIT (dv1394src_debug, "dv1394src", 0,
       "DV firewire source");
 }
@@ -211,6 +215,16 @@ gst_dv1394src_class_init (GstDV1394SrcClass * klass)
           "select one of multiple DV devices by its GUID. use a hexadecimal "
           "like 0xhhhhhhhhhhhhhhhh. (0 = no guid)", 0, G_MAXUINT64,
           DEFAULT_GUID, G_PARAM_READWRITE));
+  /**
+   * GstDV1394Src:device-name
+   *
+   * Descriptive name of the currently opened device
+   *
+   * Since: 0.10.7
+   **/
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DEVICE_NAME,
+      g_param_spec_string ("device-name", "device name",
+          "user-friendly name of the device", "Default", G_PARAM_READABLE));
 
   gstbasesrc_class->negotiate = NULL;
   gstbasesrc_class->start = gst_dv1394src_start;
@@ -240,6 +254,7 @@ gst_dv1394src_init (GstDV1394Src * dv1394src, GstDV1394SrcClass * klass)
   dv1394src->use_avc = DEFAULT_USE_AVC;
   dv1394src->guid = DEFAULT_GUID;
   dv1394src->uri = g_strdup_printf ("dv://%d", dv1394src->port);
+  dv1394src->device_name = g_strdup_printf ("Default");
 
   READ_SOCKET (dv1394src) = -1;
   WRITE_SOCKET (dv1394src) = -1;
@@ -260,6 +275,9 @@ gst_dv1394src_dispose (GObject * object)
   g_free (src->uri);
   src->uri = NULL;
 
+  g_free (src->device_name);
+  src->device_name = NULL;
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -292,6 +310,7 @@ gst_dv1394src_set_property (GObject * object, guint prop_id,
       break;
     case PROP_GUID:
       filter->guid = g_value_get_uint64 (value);
+      gst_dv1394src_update_device_name (filter);
       break;
     default:
       break;
@@ -326,12 +345,31 @@ gst_dv1394src_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_GUID:
       g_value_set_uint64 (value, filter->guid);
       break;
+    case PROP_DEVICE_NAME:
+      g_value_set_string (value, filter->device_name);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
+#ifdef HAVE_LIBIEC61883
+static GstDV1394Src *
+gst_dv1394src_from_raw1394handle (raw1394handle_t handle)
+{
+  iec61883_dv_t dv = (iec61883_dv_t) raw1394_get_userdata (handle);
+  iec61883_dv_fb_t dv_fb =
+      (iec61883_dv_fb_t) iec61883_dv_get_callback_data (dv);
+  return GST_DV1394SRC (iec61883_dv_fb_get_callback_data (dv_fb));
+}
+#else /* HAVE_LIBIEC61883 */
+static GstDV1394Src *
+gst_dv1394src_from_raw1394handle (raw1394handle_t handle)
+{
+  return GST_DV1394SRC (raw1394_get_userdata (handle));
+}
+#endif /* HAVE_LIBIEC61883 */
 
 #ifdef HAVE_LIBIEC61883
 static int
@@ -400,7 +438,7 @@ static int
 gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
     quadlet_t * data)
 {
-  GstDV1394Src *dv1394src = GST_DV1394SRC (raw1394_get_userdata (handle));
+  GstDV1394Src *dv1394src = gst_dv1394src_from_raw1394handle (handle);
 
   if (len > 16) {
     /*
@@ -538,14 +576,7 @@ gst_dv1394src_bus_reset (raw1394handle_t handle, unsigned int generation)
   gint current_device_change;
   gint i;
 
-#ifdef HAVE_LIBIEC61883
-  iec61883_dv_t dv = (iec61883_dv_t) raw1394_get_userdata (handle);
-  iec61883_dv_fb_t dv_fb =
-      (iec61883_dv_fb_t) iec61883_dv_get_callback_data (dv);
-  src = GST_DV1394SRC (iec61883_dv_fb_get_callback_data (dv_fb));
-#else
-  src = GST_DV1394SRC (raw1394_get_userdata (handle));
-#endif
+  src = gst_dv1394src_from_raw1394handle (handle);
 
   GST_INFO_OBJECT (src, "have bus reset");
 
@@ -590,20 +621,13 @@ gst_dv1394src_create (GstPushSrc * psrc, GstBuffer ** buf)
 {
   GstDV1394Src *dv1394src = GST_DV1394SRC (psrc);
   GstCaps *caps;
-  int numfds;
-
-  GST_DEBUG ("_create");
   struct pollfd pollfds[2];
 
   pollfds[0].fd = raw1394_get_fd (dv1394src->handle);
   pollfds[0].events = POLLIN | POLLERR | POLLHUP | POLLPRI;
   pollfds[1].fd = READ_SOCKET (dv1394src);
   pollfds[1].events = POLLIN | POLLERR | POLLHUP | POLLPRI;
-#ifdef HAVE_LIBIEC61883
-  numfds = 1;
-#else
-  numfds = 2;
-#endif
+
   if (dv1394src->buf) {
     /* maybe we had an error before, and there's a stale buffer? */
     gst_buffer_unref (dv1394src->buf);
@@ -611,7 +635,7 @@ gst_dv1394src_create (GstPushSrc * psrc, GstBuffer ** buf)
   }
 
   while (TRUE) {
-    int res = poll (pollfds, numfds, -1);
+    int res = poll (pollfds, 2, -1);
 
     if (res < 0) {
       if (errno == EAGAIN || errno == EINTR)
@@ -619,28 +643,20 @@ gst_dv1394src_create (GstPushSrc * psrc, GstBuffer ** buf)
       else
         goto error_while_polling;
     }
-#ifdef HAVE_LIBIEC61883
-    if ((pollfds[0].revents & POLLIN) | (pollfds[0].revents & POLLPRI)) {
-      res = raw1394_loop_iterate (dv1394src->handle);
-      if (dv1394src->buf)
-        break;
-    }
-#else
-    if (pollfds[1].revents) {
+    if (G_UNLIKELY (pollfds[1].revents)) {
       char command;
 
       if (pollfds[1].revents & POLLIN)
         READ_COMMAND (dv1394src, command, res);
 
       goto told_to_stop;
-    } else if (pollfds[0].revents & POLLIN) {
+    } else if (G_LIKELY (pollfds[0].revents & POLLIN)) {
       /* shouldn't block in theory */
       raw1394_loop_iterate (dv1394src->handle);
 
       if (dv1394src->buf)
         break;
     }
-#endif
   }
 
   g_assert (dv1394src->buf);
@@ -658,13 +674,11 @@ error_while_polling:
     GST_ELEMENT_ERROR (dv1394src, RESOURCE, READ, (NULL), GST_ERROR_SYSTEM);
     return GST_FLOW_UNEXPECTED;
   }
-#ifndef HAVE_LIBIEC61883
 told_to_stop:
   {
     GST_DEBUG_OBJECT (dv1394src, "told to stop, shutting down");
     return GST_FLOW_WRONG_STATE;
   }
-#endif
 }
 
 static int
@@ -730,8 +744,11 @@ gst_dv1394src_discover_avc_node (GstDV1394Src * src)
           src->guid = rom1394_get_guid (handle, i);
           g_free (src->uri);
           src->uri = g_strdup_printf ("dv://%d", src->port);
+          g_free (src->device_name);
+          src->device_name = g_strdup (rom_dir.label);
           break;
         }
+        rom1394_free_directory (&rom_dir);
       }
     }
   next:
@@ -784,6 +801,8 @@ gst_dv1394src_start (GstBaseSrc * bsrc)
   if (!src->handle)
     goto cannot_set_port;
 
+  raw1394_set_userdata (src->handle, src);
+  raw1394_set_bus_reset_handler (src->handle, gst_dv1394src_bus_reset);
 
 #ifdef HAVE_LIBIEC61883
   if ((src->iec61883dv =
@@ -792,11 +811,9 @@ gst_dv1394src_start (GstBaseSrc * bsrc)
     goto cannot_initialise_dv;
 
 #else
-  raw1394_set_userdata (src->handle, src);
   raw1394_set_iso_handler (src->handle, src->channel,
       gst_dv1394src_iso_receive);
 #endif
-  raw1394_set_bus_reset_handler (src->handle, gst_dv1394src_bus_reset);
 
   GST_DEBUG_OBJECT (src, "successfully opened up 1394 connection");
   src->connected = TRUE;
@@ -810,11 +827,19 @@ gst_dv1394src_start (GstBaseSrc * bsrc)
 #endif
 
   if (src->use_avc) {
+    raw1394handle_t avc_handle = raw1394_new_handle_on_port (src->port);
+
     /* start the VCR */
-    if (!avc1394_vcr_is_recording (src->handle, src->avc_node)
-        && avc1394_vcr_is_playing (src->handle, src->avc_node)
-        != AVC1394_VCR_OPERAND_PLAY_FORWARD)
-      avc1394_vcr_play (src->handle, src->avc_node);
+    if (avc_handle) {
+      if (!avc1394_vcr_is_recording (avc_handle, src->avc_node)
+          && avc1394_vcr_is_playing (avc_handle, src->avc_node)
+          != AVC1394_VCR_OPERAND_PLAY_FORWARD)
+        avc1394_vcr_play (avc_handle, src->avc_node);
+      raw1394_destroy_handle (avc_handle);
+    } else {
+      GST_WARNING_OBJECT (src, "Starting VCR via avc1394 failed: %s",
+          g_strerror (errno));
+    }
   }
 
   return TRUE;
@@ -843,6 +868,8 @@ no_handle:
   }
 no_ports:
   {
+    raw1394_destroy_handle (src->handle);
+    src->handle = NULL;
     GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND, (NULL),
         ("no ports available for raw1394"));
     return FALSE;
@@ -855,6 +882,12 @@ cannot_set_port:
   }
 cannot_start:
   {
+    raw1394_destroy_handle (src->handle);
+    src->handle = NULL;
+#ifdef HAVE_LIBIEC61883
+    iec61883_dv_fb_close (src->iec61883dv);
+    src->iec61883dv = NULL;
+#endif
     GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
         ("can't start 1394 iso receive"));
     return FALSE;
@@ -862,6 +895,8 @@ cannot_start:
 #ifdef HAVE_LIBIEC61883
 cannot_initialise_dv:
   {
+    raw1394_destroy_handle (src->handle);
+    src->handle = NULL;
     GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
         ("can't initialise iec61883 dv"));
     return FALSE;
@@ -883,17 +918,23 @@ gst_dv1394src_stop (GstBaseSrc * bsrc)
 #else
   raw1394_stop_iso_rcv (src->handle, src->channel);
 #endif
-  if (src->use_avc) {
-    /* pause the VCR */
-    if (!avc1394_vcr_is_recording (src->handle, src->avc_node)
-        && (avc1394_vcr_is_playing (src->handle, src->avc_node)
-            != AVC1394_VCR_OPERAND_PLAY_FORWARD_PAUSE))
-      avc1394_vcr_pause (src->handle, src->avc_node);
-  }
 
-  if (src->use_avc)
-    /* stop the VCR */
-    avc1394_vcr_stop (src->handle, src->avc_node);
+  if (src->use_avc) {
+    raw1394handle_t avc_handle = raw1394_new_handle_on_port (src->port);
+
+    /* pause and stop the VCR */
+    if (avc_handle) {
+      if (!avc1394_vcr_is_recording (avc_handle, src->avc_node)
+          && (avc1394_vcr_is_playing (avc_handle, src->avc_node)
+              != AVC1394_VCR_OPERAND_PLAY_FORWARD_PAUSE))
+        avc1394_vcr_pause (avc_handle, src->avc_node);
+      avc1394_vcr_stop (avc_handle, src->avc_node);
+      raw1394_destroy_handle (avc_handle);
+    } else {
+      GST_WARNING_OBJECT (src, "Starting VCR via avc1394 failed: %s",
+          g_strerror (errno));
+    }
+  }
 
   raw1394_destroy_handle (src->handle);
 
@@ -1029,6 +1070,58 @@ not_supported:
   }
 }
 
+static void
+gst_dv1394src_update_device_name (GstDV1394Src * src)
+{
+  raw1394handle_t handle;
+  gint portcount, port, nodecount, node;
+  rom1394_directory directory;
+
+  g_free (src->device_name);
+  src->device_name = NULL;
+
+  GST_LOG_OBJECT (src, "updating device name for current GUID");
+
+  handle = raw1394_new_handle ();
+
+  if (handle == NULL)
+    goto gethandle_failed;
+
+  portcount = raw1394_get_port_info (handle, NULL, 0);
+  for (port = 0; port < portcount; port++) {
+    if (raw1394_set_port (handle, port) >= 0) {
+      nodecount = raw1394_get_nodecount (handle);
+      for (node = 0; node < nodecount; node++) {
+        if (src->guid == rom1394_get_guid (handle, node)) {
+          if (rom1394_get_directory (handle, node, &directory) >= 0) {
+            g_free (src->device_name);
+            src->device_name = g_strdup (directory.label);
+            rom1394_free_directory (&directory);
+            goto done;
+          } else {
+            GST_WARNING ("error reading rom directory for node %d", node);
+          }
+        }
+      }
+    }
+  }
+
+  src->device_name = g_strdup ("Unknown");      /* FIXME: translate? */
+
+done:
+
+  raw1394_destroy_handle (handle);
+  return;
+
+/* ERRORS */
+gethandle_failed:
+  {
+    GST_WARNING ("failed to get raw1394 handle: %s", g_strerror (errno));
+    src->device_name = g_strdup ("Unknown");    /* FIXME: translate? */
+    return;
+  }
+}
+
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
 static guint
@@ -1069,7 +1162,7 @@ gst_dv1394src_uri_set_uri (GstURIHandler * handler, const gchar * uri)
   g_free (protocol);
 
   location = gst_uri_get_location (uri);
-  if (location && (!(location == "")))
+  if (location && *location != '\0')
     gst_dv1394src->port = strtol (location, NULL, 10);
   else
     gst_dv1394src->port = DEFAULT_PORT;
