@@ -78,10 +78,11 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include <string.h>
+#include <math.h>
 #include <gst/gst.h>
 #include <gst/audio/audio.h>
 #include "gstlevel.h"
-#include "math.h"
 
 GST_DEBUG_CATEGORY_STATIC (level_debug);
 #define GST_CAT_DEFAULT level_debug
@@ -92,7 +93,7 @@ static const GstElementDetails level_details = GST_ELEMENT_DETAILS ("Level",
     "Thomas Vander Stichele <thomas at apestaart dot org>");
 
 static GstStaticPadTemplate sink_template_factory =
-GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-int, "
@@ -100,11 +101,16 @@ GST_STATIC_PAD_TEMPLATE ("sink",
         "channels = (int) [ 1, MAX ], "
         "endianness = (int) BYTE_ORDER, "
         "width = (int) { 8, 16 }, "
-        "depth = (int) { 8, 16 }, " "signed = (boolean) true")
+        "depth = (int) { 8, 16 }, "
+        "signed = (boolean) true; "
+        "audio/x-raw-float, "
+        "rate = (int) [ 1, MAX ], "
+        "channels = (int) [ 1, MAX ], "
+        "endianness = (int) BYTE_ORDER, " "width = (int) {32, 64} ")
     );
 
 static GstStaticPadTemplate src_template_factory =
-GST_STATIC_PAD_TEMPLATE ("src",
+    GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-int, "
@@ -112,7 +118,12 @@ GST_STATIC_PAD_TEMPLATE ("src",
         "channels = (int) [ 1, MAX ], "
         "endianness = (int) BYTE_ORDER, "
         "width = (int) { 8, 16 }, "
-        "depth = (int) { 8, 16 }, " "signed = (boolean) true")
+        "depth = (int) { 8, 16 }, "
+        "signed = (boolean) true; "
+        "audio/x-raw-float, "
+        "rate = (int) [ 1, MAX ], "
+        "channels = (int) [ 1, MAX ], "
+        "endianness = (int) BYTE_ORDER, " "width = (int) {32, 64} ")
     );
 
 enum
@@ -131,10 +142,11 @@ static void gst_level_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_level_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
-static void gst_level_dispose (GObject * obj);
+static void gst_level_finalize (GObject * obj);
 
 static gboolean gst_level_set_caps (GstBaseTransform * trans, GstCaps * in,
     GstCaps * out);
+static gboolean gst_level_start (GstBaseTransform * trans);
 static GstFlowReturn gst_level_transform_ip (GstBaseTransform * trans,
     GstBuffer * in);
 
@@ -154,37 +166,35 @@ gst_level_base_init (gpointer g_class)
 static void
 gst_level_class_init (GstLevelClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstBaseTransformClass *trans_class;
-
-  gobject_class = (GObjectClass *) klass;
-  trans_class = (GstBaseTransformClass *) klass;
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GstBaseTransformClass *trans_class = GST_BASE_TRANSFORM_CLASS (klass);
 
   gobject_class->set_property = gst_level_set_property;
   gobject_class->get_property = gst_level_get_property;
-  gobject_class->dispose = gst_level_dispose;
+  gobject_class->finalize = gst_level_finalize;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SIGNAL_LEVEL,
+  g_object_class_install_property (gobject_class, PROP_SIGNAL_LEVEL,
       g_param_spec_boolean ("message", "mesage",
           "Post a level message for each passed interval",
           TRUE, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SIGNAL_INTERVAL,
+  g_object_class_install_property (gobject_class, PROP_SIGNAL_INTERVAL,
       g_param_spec_uint64 ("interval", "Interval",
           "Interval of time between message posts (in nanoseconds)",
           1, G_MAXUINT64, GST_SECOND / 10, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PEAK_TTL,
+  g_object_class_install_property (gobject_class, PROP_PEAK_TTL,
       g_param_spec_uint64 ("peak-ttl", "Peak TTL",
           "Time To Live of decay peak before it falls back (in nanoseconds)",
           0, G_MAXUINT64, GST_SECOND / 10 * 3, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PEAK_FALLOFF,
+  g_object_class_install_property (gobject_class, PROP_PEAK_FALLOFF,
       g_param_spec_double ("peak-falloff", "Peak Falloff",
           "Decay rate of decay peak after TTL (in dB/sec)",
           0.0, G_MAXDOUBLE, 10.0, G_PARAM_READWRITE));
 
   GST_DEBUG_CATEGORY_INIT (level_debug, "level", 0, "Level calculation");
 
-  trans_class->set_caps = gst_level_set_caps;
-  trans_class->transform_ip = gst_level_transform_ip;
+  trans_class->set_caps = GST_DEBUG_FUNCPTR (gst_level_set_caps);
+  trans_class->start = GST_DEBUG_FUNCPTR (gst_level_start);
+  trans_class->transform_ip = GST_DEBUG_FUNCPTR (gst_level_transform_ip);
   trans_class->passthrough_on_same_caps = TRUE;
 }
 
@@ -203,10 +213,14 @@ gst_level_init (GstLevel * filter, GstLevelClass * g_class)
   filter->decay_peak_falloff = 10.0;    /* dB falloff (/sec) */
 
   filter->message = TRUE;
+
+  filter->process = NULL;
+
+  gst_base_transform_set_gap_aware (GST_BASE_TRANSFORM (filter), TRUE);
 }
 
 static void
-gst_level_dispose (GObject * obj)
+gst_level_finalize (GObject * obj)
 {
   GstLevel *filter = GST_LEVEL (obj);
 
@@ -224,7 +238,7 @@ gst_level_dispose (GObject * obj)
   filter->decay_peak_base = NULL;
   filter->decay_peak_age = NULL;
 
-  G_OBJECT_CLASS (parent_class)->dispose (obj);
+  G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -248,6 +262,7 @@ gst_level_set_property (GObject * object, guint prop_id,
       filter->decay_peak_falloff = g_value_get_double (value);
       break;
     default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
@@ -277,6 +292,84 @@ gst_level_get_property (GObject * object, guint prop_id,
   }
 }
 
+
+/* process one (interleaved) channel of incoming samples
+ * calculate square sum of samples
+ * normalize and average over number of samples
+ * returns a normalized cumulative square value, which can be averaged
+ * to return the average power as a double between 0 and 1
+ * also returns the normalized peak power (square of the highest amplitude)
+ *
+ * caller must assure num is a multiple of channels
+ * samples for multiple channels are interleaved
+ * input sample data enters in *in_data as 8 or 16 bit data
+ * this filter only accepts signed audio data, so mid level is always 0
+ *
+ * for 16 bit, this code considers the non-existant 32768 value to be
+ * full-scale; so 32767 will not map to 1.0
+ */
+
+#define DEFINE_INT_LEVEL_CALCULATOR(TYPE, RESOLUTION)                         \
+static void inline                                                          \
+gst_level_calculate_##TYPE (gpointer data, guint num, guint channels,         \
+                            gdouble *NCS, gdouble *NPS)                       \
+{                                                                             \
+  TYPE * in = (TYPE *)data;                                                   \
+  register guint j;                                                           \
+  gdouble squaresum = 0.0;           /* square sum of the integer samples */  \
+  register gdouble square = 0.0;     /* Square */                             \
+  register gdouble peaksquare = 0.0; /* Peak Square Sample */                 \
+  gdouble normalizer;               /* divisor to get a [-1.0, 1.0] range */  \
+                                                                              \
+  /* *NCS = 0.0; Normalized Cumulative Square */                              \
+  /* *NPS = 0.0; Normalized Peask Square */                                   \
+                                                                              \
+  normalizer = (gdouble) (1 << (RESOLUTION * 2));                             \
+                                                                              \
+  /* oil_squaresum_f64(&squaresum,in,num); */                                 \
+  for (j = 0; j < num; j += channels)                                         \
+  {                                                                           \
+    square = ((gdouble) in[j]) * in[j];                                       \
+    if (square > peaksquare) peaksquare = square;                             \
+    squaresum += square;                                                      \
+  }                                                                           \
+                                                                              \
+  *NCS = squaresum / normalizer;                                              \
+  *NPS = peaksquare / normalizer;                                             \
+}
+
+DEFINE_INT_LEVEL_CALCULATOR (gint16, 15);
+DEFINE_INT_LEVEL_CALCULATOR (gint8, 7);
+
+#define DEFINE_FLOAT_LEVEL_CALCULATOR(TYPE)                                   \
+static void inline                                                            \
+gst_level_calculate_##TYPE (gpointer data, guint num, guint channels,         \
+                            gdouble *NCS, gdouble *NPS)                       \
+{                                                                             \
+  TYPE * in = (TYPE *)data;                                                   \
+  register guint j;                                                           \
+  gdouble squaresum = 0.0;           /* square sum of the integer samples */  \
+  register gdouble square = 0.0;     /* Square */                             \
+  register gdouble peaksquare = 0.0; /* Peak Square Sample */                 \
+                                                                              \
+  /* *NCS = 0.0; Normalized Cumulative Square */                              \
+  /* *NPS = 0.0; Normalized Peask Square */                                   \
+                                                                              \
+  /* oil_squaresum_f64(&squaresum,in,num); */                                 \
+  for (j = 0; j < num; j += channels)                                         \
+  {                                                                           \
+    square = ((gdouble) in[j]) * in[j];                                       \
+    if (square > peaksquare) peaksquare = square;                             \
+    squaresum += square;                                                      \
+  }                                                                           \
+                                                                              \
+  *NCS = squaresum;                                                           \
+  *NPS = peaksquare;                                                          \
+}
+
+DEFINE_FLOAT_LEVEL_CALCULATOR (gfloat);
+DEFINE_FLOAT_LEVEL_CALCULATOR (gdouble);
+
 static gint
 structure_get_int (GstStructure * structure, const gchar * field)
 {
@@ -291,18 +384,40 @@ structure_get_int (GstStructure * structure, const gchar * field)
 static gboolean
 gst_level_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
 {
-  GstLevel *filter;
+  GstLevel *filter = GST_LEVEL (trans);
+  const gchar *mimetype;
   GstStructure *structure;
   int i;
 
-  filter = GST_LEVEL (trans);
-
-  filter->num_frames = 0;
-
   structure = gst_caps_get_structure (in, 0);
   filter->rate = structure_get_int (structure, "rate");
   filter->width = structure_get_int (structure, "width");
   filter->channels = structure_get_int (structure, "channels");
+  mimetype = gst_structure_get_name (structure);
+
+  /* FIXME: set calculator func depending on caps */
+  filter->process = NULL;
+  if (strcmp (mimetype, "audio/x-raw-int") == 0) {
+    GST_DEBUG_OBJECT (filter, "use int: %u", filter->width);
+    switch (filter->width) {
+      case 8:
+        filter->process = gst_level_calculate_gint8;
+        break;
+      case 16:
+        filter->process = gst_level_calculate_gint16;
+        break;
+    }
+  } else if (strcmp (mimetype, "audio/x-raw-float") == 0) {
+    GST_DEBUG_OBJECT (filter, "use float, %u", filter->width);
+    switch (filter->width) {
+      case 32:
+        filter->process = gst_level_calculate_gfloat;
+        break;
+      case 64:
+        filter->process = gst_level_calculate_gdouble;
+        break;
+    }
+  }
 
   /* allocate channel variable arrays */
   g_free (filter->CS);
@@ -328,52 +443,16 @@ gst_level_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
   return TRUE;
 }
 
-/* process one (interleaved) channel of incoming samples
- * calculate square sum of samples
- * normalize and average over number of samples
- * returns a normalized cumulative square value, which can be averaged
- * to return the average power as a double between 0 and 1
- * also returns the normalized peak power (square of the highest amplitude)
- *
- * caller must assure num is a multiple of channels
- * samples for multiple channels are interleaved
- * input sample data enters in *in_data as 8 or 16 bit data
- * this filter only accepts signed audio data, so mid level is always 0
- *
- * for 16 bit, this code considers the non-existant 32768 value to be
- * full-scale; so 32767 will not map to 1.0
- */
+static gboolean
+gst_level_start (GstBaseTransform * trans)
+{
+  GstLevel *filter = GST_LEVEL (trans);
 
-#define DEFINE_LEVEL_CALCULATOR(TYPE, RESOLUTION)                             \
-static void inline                                                            \
-gst_level_calculate_##TYPE (TYPE * in, guint num, gint channels,              \
-                            double *NCS, double *NPS)                         \
-{                                                                             \
-  register int j;                                                             \
-  double squaresum = 0.0;           /* square sum of the integer samples */   \
-  register double square = 0.0;     /* Square */                              \
-  register double peaksquare = 0.0; /* Peak Square Sample */                  \
-  gdouble normalizer;               /* divisor to get a [-1.0, 1.0] range */  \
-                                                                              \
-  *NCS = 0.0;                       /* Normalized Cumulative Square */        \
-  *NPS = 0.0;                       /* Normalized Peask Square */             \
-                                                                              \
-  normalizer = (double) (1 << (RESOLUTION * 2));                              \
-                                                                              \
-  for (j = 0; j < num; j += channels)                                         \
-  {                                                                           \
-    square = ((double) in[j]) * in[j];                                        \
-    if (square > peaksquare) peaksquare = square;                             \
-    squaresum += square;                                                      \
-  }                                                                           \
-                                                                              \
-  *NCS = squaresum / normalizer;                                              \
-  *NPS = peaksquare / normalizer;                                             \
+  filter->num_frames = 0;
+
+  return TRUE;
 }
 
-DEFINE_LEVEL_CALCULATOR (gint16, 15);
-DEFINE_LEVEL_CALCULATOR (gint8, 7);
-
 static GstMessage *
 gst_level_message_new (GstLevel * l, GstClockTime endtime)
 {
@@ -425,47 +504,39 @@ static GstFlowReturn
 gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
 {
   GstLevel *filter;
-  gpointer in_data;
-  double CS = 0.0;
-  gint num_frames = 0;
-  gint num_int_samples = 0;     /* number of interleaved samples
+  guint8 *in_data;
+  double CS;
+  guint i;
+  guint num_frames = 0;
+  guint num_int_samples = 0;    /* number of interleaved samples
                                  * ie. total count for all channels combined */
-  gint i;
 
   filter = GST_LEVEL (trans);
 
-  for (i = 0; i < filter->channels; ++i)
-    filter->peak[i] = 0.0;
-
   in_data = GST_BUFFER_DATA (in);
   num_int_samples = GST_BUFFER_SIZE (in) / (filter->width / 8);
 
+  GST_LOG_OBJECT (filter, "analyzing %u sample frames at ts %" GST_TIME_FORMAT,
+      num_int_samples, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (in)));
+
   g_return_val_if_fail (num_int_samples % filter->channels == 0,
       GST_FLOW_ERROR);
 
   num_frames = num_int_samples / filter->channels;
 
   for (i = 0; i < filter->channels; ++i) {
-    CS = 0.0;
-    switch (filter->width) {
-      case 16:
-        gst_level_calculate_gint16 (((gint16 *) in_data) + i, num_int_samples,
-            filter->channels, &CS, &filter->peak[i]);
-        break;
-      case 8:
-        gst_level_calculate_gint8 (((gint8 *) in_data) + i, num_int_samples,
-            filter->channels, &CS, &filter->peak[i]);
-        break;
+    if (!GST_BUFFER_FLAG_IS_SET (in, GST_BUFFER_FLAG_GAP)) {
+      filter->process (in_data, num_int_samples, filter->channels, &CS,
+          &filter->peak[i]);
+      GST_LOG_OBJECT (filter,
+          "channel %d, cumulative sum %f, peak %f, over %d samples/%d channels",
+          i, CS, filter->peak[i], num_int_samples, filter->channels);
+      filter->CS[i] += CS;
+    } else {
+      filter->peak[i] = 0.0;
     }
-    GST_LOG_OBJECT (filter,
-        "channel %d, cumulative sum %f, peak %f, over %d samples/%d channels",
-        i, CS, filter->peak[i], num_int_samples, filter->channels);
-    filter->CS[i] += CS;
-  }
+    in_data += (filter->width / 8);
 
-  filter->num_frames += num_frames;
-
-  for (i = 0; i < filter->channels; ++i) {
     filter->decay_peak_age[i] +=
         GST_FRAMES_TO_CLOCK_TIME (num_frames, filter->rate);
     GST_LOG_OBJECT (filter, "filter peak info [%d]: decay peak %f, age %"
@@ -513,14 +584,16 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
     }
   }
 
+  filter->num_frames += num_frames;
+
   /* do we need to message ? */
   if (filter->num_frames >=
       GST_CLOCK_TIME_TO_FRAMES (filter->interval, filter->rate)) {
     if (filter->message) {
       GstMessage *m;
-      GstClockTime endtime;
-
-      endtime = GST_BUFFER_TIMESTAMP (in)
+      GstClockTime endtime =
+          gst_segment_to_stream_time (&trans->segment, GST_FORMAT_TIME,
+          GST_BUFFER_TIMESTAMP (in))
           + GST_FRAMES_TO_CLOCK_TIME (num_frames, filter->rate);
 
       m = gst_level_message_new (filter, endtime);
