@@ -22,6 +22,7 @@
 #endif
 
 #include "gstrtpdepay.h"
+#include "gstrtpac3depay.h"
 #include "gstrtpilbcdepay.h"
 #include "gstrtpilbcpay.h"
 #include "gstrtppcmupay.h"
@@ -34,14 +35,22 @@
 #include "gstrtpamrdepay.h"
 #include "gstrtpmpapay.h"
 #include "gstrtpmpadepay.h"
+#include "gstrtpmpvdepay.h"
+#include "gstrtpmpvpay.h"
 #include "gstrtph263pdepay.h"
 #include "gstrtph263ppay.h"
+#include "gstrtph263depay.h"
 #include "gstrtph263pay.h"
 #include "gstrtph264depay.h"
+#include "gstrtph264pay.h"
+#include "gstrtpL16depay.h"
+#include "gstrtpL16pay.h"
 #include "gstasteriskh263.h"
 #include "gstrtpmp2tdepay.h"
+#include "gstrtpmp2tpay.h"
 #include "gstrtpmp4vdepay.h"
 #include "gstrtpmp4vpay.h"
+#include "gstrtpmp4adepay.h"
 #include "gstrtpmp4gdepay.h"
 #include "gstrtpmp4gpay.h"
 #include "gstrtpspeexpay.h"
@@ -58,6 +67,9 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_depay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_ac3_depay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_ilbc_pay_plugin_init (plugin))
     return FALSE;
 
@@ -94,30 +106,54 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_mpa_pay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_mpv_depay_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rtp_mpv_pay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_h263p_pay_plugin_init (plugin))
     return FALSE;
 
   if (!gst_rtp_h263p_depay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_h263_depay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_h263_pay_plugin_init (plugin))
     return FALSE;
 
   if (!gst_rtp_h264_depay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_h264_pay_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rtp_L16_pay_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rtp_L16_depay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_asteriskh263_plugin_init (plugin))
     return FALSE;
 
   if (!gst_rtp_mp2t_depay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_mp2t_pay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_mp4v_pay_plugin_init (plugin))
     return FALSE;
 
   if (!gst_rtp_mp4v_depay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_mp4a_depay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_mp4g_depay_plugin_init (plugin))
     return FALSE;
 
