@@ -30,47 +30,63 @@
 static const GstElementDetails gst_rtp_h263pdepay_details =
 GST_ELEMENT_DETAILS ("RTP packet depayloader",
     "Codec/Depayloader/Network",
-    "Extracts H263+ video from RTP packets (RFC 2429)",
+    "Extracts H263/+/++ video from RTP packets (RFC 4629)",
     "Wim Taymans <wim@fluendo.com>");
 
-/* RtpH263PDepay signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
-enum
-{
-  ARG_0,
-  ARG_FREQUENCY
-};
-
 static GstStaticPadTemplate gst_rtp_h263p_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-h263, "
-        "variant = (string) \"itu\", " "h263version = (string) \"h263p\"")
+    GST_STATIC_CAPS ("video/x-h263, " "variant = (string) \"itu\" ")
     );
 
 static GstStaticPadTemplate gst_rtp_h263p_depay_sink_template =
-GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
-        "clock-rate = (int) 90000, " "encoding-name = (string) \"H263-1998\"")
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) 90000, " "encoding-name = (string) \"H263-1998\"; "
+        /* optional params */
+        /* NOTE all optional SDP params must be strings in the caps */
+        /*
+           "sqcif = (string) [1, 32], "
+           "qcif = (string) [1, 32], "
+           "cif = (string) [1, 32], "
+           "cif4 = (string) [1, 32], "
+           "cif16 = (string) [1, 32], "
+           "custom = (string) ANY, "
+           "f = (string) {0, 1},"
+           "i = (string) {0, 1},"
+           "j = (string) {0, 1},"
+           "t = (string) {0, 1},"
+           "k = (string) {1, 2, 3, 4},"
+           "n = (string) {1, 2, 3, 4},"
+           "p = (string) ANY,"
+           "par = (string) ANY, "
+           "cpcf = (string) ANY, "
+           "bpp = (string) [0, 65536], "
+           "hrd = (string) {0, 1}; "
+         */
+        "application/x-rtp, "
+        "media = (string) \"video\", "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) 90000, " "encoding-name = (string) \"H263-2000\" "
+        /* optional params */
+        /* NOTE all optional SDP params must be strings in the caps */
+        /*
+           "profile = (string) [0, 10], "
+           "level = (string) {10, 20, 30, 40, 45, 50, 60, 70}, "
+           "interlace = (string) {0, 1};"
+         */
+    )
     );
 
 GST_BOILERPLATE (GstRtpH263PDepay, gst_rtp_h263p_depay, GstBaseRTPDepayload,
     GST_TYPE_BASE_RTP_DEPAYLOAD);
 
 static void gst_rtp_h263p_depay_finalize (GObject * object);
-static void gst_rtp_h263p_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_rtp_h263p_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
 
 static GstStateChangeReturn gst_rtp_h263p_depay_change_state (GstElement *
     element, GstStateChange transition);
@@ -112,9 +128,6 @@ gst_rtp_h263p_depay_class_init (GstRtpH263PDepayClass * klass)
 
   gobject_class->finalize = gst_rtp_h263p_depay_finalize;
 
-  gobject_class->set_property = gst_rtp_h263p_depay_set_property;
-  gobject_class->get_property = gst_rtp_h263p_depay_get_property;
-
   gstelement_class->change_state = gst_rtp_h263p_depay_change_state;
 }
 
@@ -138,23 +151,88 @@ gst_rtp_h263p_depay_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-// only on the sink
 gboolean
 gst_rtp_h263p_depay_setcaps (GstBaseRTPDepayload * filter, GstCaps * caps)
 {
-
+  GstCaps *srccaps = NULL;
   GstStructure *structure = gst_caps_get_structure (caps, 0);
-  gint clock_rate = 90000;      // default
-
-  if (gst_structure_has_field (structure, "clock-rate")) {
-    gst_structure_get_int (structure, "clock-rate", &clock_rate);
-  }
+  gint clock_rate = 90000;      /* default */
+  const gchar *encoding_name = NULL;
 
+  gst_structure_get_int (structure, "clock-rate", &clock_rate);
   filter->clock_rate = clock_rate;
 
-  return TRUE;
-}
+  encoding_name = gst_structure_get_string (structure, "encoding-name");
+  if (encoding_name == NULL)
+    goto no_encoding_name;
 
+  if (g_ascii_strcasecmp (encoding_name, "H263-2000") == 0) {
+    /* always h263++ */
+    srccaps = gst_caps_new_simple ("video/x-h263",
+        "variant", G_TYPE_STRING, "itu",
+        "h263version", G_TYPE_STRING, "h263pp", NULL);
+  } else if (g_ascii_strcasecmp (encoding_name, "H263-1998") == 0) {
+    /* this can be H263 or H263+ depending on defined appendixes in the optional
+     * SDP params */
+    const gchar *F, *I, *J, *T, *K, *N, *P;
+    gboolean is_h263p = FALSE;
+
+    F = gst_structure_get_string (structure, "f");
+    if (F)
+      if (g_strcasecmp (F, "1") == 0)
+        is_h263p = TRUE;
+    I = gst_structure_get_string (structure, "i");
+    if (I)
+      if (g_strcasecmp (I, "1") == 0)
+        is_h263p = TRUE;
+    J = gst_structure_get_string (structure, "j");
+    if (J)
+      if (g_strcasecmp (J, "1") == 0)
+        is_h263p = TRUE;
+    T = gst_structure_get_string (structure, "t");
+    if (T)
+      if (g_strcasecmp (T, "1") == 0)
+        is_h263p = TRUE;
+    K = gst_structure_get_string (structure, "k");
+    if (K)
+      is_h263p = TRUE;
+    N = gst_structure_get_string (structure, "n");
+    if (N)
+      is_h263p = TRUE;
+    P = gst_structure_get_string (structure, "p");
+    if (P)
+      is_h263p = TRUE;
+
+    if (is_h263p) {
+      srccaps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "itu",
+          "h263version", G_TYPE_STRING, "h263p", NULL);
+    } else {
+      srccaps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "itu",
+          "h263version", G_TYPE_STRING, "h263", NULL);
+    }
+  }
+  if (!srccaps)
+    goto no_caps;
+
+  gst_pad_set_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (filter), srccaps);
+  gst_caps_unref (srccaps);
+
+  return TRUE;
+
+  /* ERRORS */
+no_encoding_name:
+  {
+    GST_ERROR_OBJECT (filter, "no encoding-name");
+    return FALSE;
+  }
+no_caps:
+  {
+    GST_ERROR_OBJECT (filter, "invalid encoding-name");
+    return FALSE;
+  }
+}
 
 static GstBuffer *
 gst_rtp_h263p_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
@@ -163,17 +241,16 @@ gst_rtp_h263p_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   GstRtpH263PDepay *rtph263pdepay;
   GstBuffer *outbuf;
 
-  /* GstRTPPayload pt; */
-
   rtph263pdepay = GST_RTP_H263P_DEPAY (depayload);
 
   if (!gst_rtp_buffer_validate (buf))
     goto bad_packet;
 
-  /*
-     if ((pt = gst_rtp_buffer_get_payload_type (buf)) != 0)
-     goto bad_payload;
-   */
+  /* flush remaining data on discont */
+  if (GST_BUFFER_IS_DISCONT (buf)) {
+    gst_adapter_clear (rtph263pdepay->adapter);
+    rtph263pdepay->wait_start = TRUE;
+  }
 
   {
     gint payload_len;
@@ -181,7 +258,7 @@ gst_rtp_h263p_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     gboolean P, V, M;
     guint32 timestamp;
     guint header_len;
-    guint8 PLEN;
+    guint8 PLEN, PEBIT;
 
     payload_len = gst_rtp_buffer_get_payload_len (buf);
     payload = gst_rtp_buffer_get_payload (buf);
@@ -189,9 +266,17 @@ gst_rtp_h263p_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     header_len = 2;
 
     M = gst_rtp_buffer_get_marker (buf);
+
+    /*  0                   1
+     *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
+     * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+     * |   RR    |P|V|   PLEN    |PEBIT|
+     * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+     */
     P = (payload[0] & 0x04) == 0x04;
     V = (payload[0] & 0x02) == 0x02;
     PLEN = ((payload[0] & 0x1) << 5) | (payload[1] >> 3);
+    PEBIT = payload[1] & 0x7;
 
     if (V) {
       header_len++;
@@ -201,13 +286,18 @@ gst_rtp_h263p_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     }
 
     if (P) {
+      rtph263pdepay->wait_start = FALSE;
       header_len -= 2;
       payload[header_len] = 0;
       payload[header_len + 1] = 0;
     }
 
+    if (rtph263pdepay->wait_start)
+      goto waiting_start;
+
     /* FIXME do not ignore the VRC header (See RFC 2429 section 4.2) */
-    /* strip off header */
+    /* FIXME actually use the RTP picture header when it is lost in the network */
+    /* for now strip off header */
     payload += header_len;
     payload_len -= header_len;
 
@@ -215,31 +305,25 @@ gst_rtp_h263p_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
     if (M) {
       /* frame is completed: append to previous, push it out */
-      guint len;
+      guint len, padlen;
       guint avail;
-      guint8 *data;
 
       avail = gst_adapter_available (rtph263pdepay->adapter);
 
       len = avail + payload_len;
-      outbuf = gst_buffer_new_and_alloc (len + (len % 4) + 4);
-      memset (GST_BUFFER_DATA (outbuf) + len, 0, (len % 4) + 4);
+      padlen = (len % 4) + 4;
+      outbuf = gst_buffer_new_and_alloc (len + padlen);
+      memset (GST_BUFFER_DATA (outbuf) + len, 0, padlen);
       GST_BUFFER_SIZE (outbuf) = len;
 
       /* prepend previous data */
       if (avail > 0) {
-        data = (guint8 *) gst_adapter_peek (rtph263pdepay->adapter, avail);
-        memcpy (GST_BUFFER_DATA (outbuf), data, avail);
+        gst_adapter_copy (rtph263pdepay->adapter, GST_BUFFER_DATA (outbuf), 0,
+            avail);
         gst_adapter_flush (rtph263pdepay->adapter, avail);
       }
       memcpy (GST_BUFFER_DATA (outbuf) + avail, payload, payload_len);
 
-      GST_BUFFER_TIMESTAMP (outbuf) =
-          timestamp * GST_SECOND / depayload->clock_rate;
-
-      gst_buffer_set_caps (outbuf,
-          (GstCaps *) gst_pad_get_pad_template_caps (depayload->srcpad));
-
       return outbuf;
 
     } else {
@@ -249,58 +333,20 @@ gst_rtp_h263p_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
       memcpy (GST_BUFFER_DATA (outbuf), payload, payload_len);
 
       gst_adapter_push (rtph263pdepay->adapter, outbuf);
-
     }
-
   }
-
   return NULL;
 
 bad_packet:
   {
     GST_ELEMENT_WARNING (rtph263pdepay, STREAM, DECODE,
         ("Packet did not validate"), (NULL));
-
     return NULL;
   }
-  /*
-     bad_payload:
-     {
-     GST_DEBUG ("Unexpected payload type %u", pt);
-
-     gst_buffer_unref (buf);
-     return GST_FLOW_ERROR;
-     }
-   */
-}
-
-static void
-gst_rtp_h263p_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstRtpH263PDepay *rtph263pdepay;
-
-  rtph263pdepay = GST_RTP_H263P_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_rtp_h263p_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstRtpH263PDepay *rtph263pdepay;
-
-  rtph263pdepay = GST_RTP_H263P_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
+waiting_start:
+  {
+    GST_DEBUG_OBJECT (rtph263pdepay, "waiting for picture start");
+    return NULL;
   }
 }
 
@@ -318,6 +364,7 @@ gst_rtp_h263p_depay_change_state (GstElement * element,
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       gst_adapter_clear (rtph263pdepay->adapter);
+      rtph263pdepay->wait_start = TRUE;
       break;
     default:
       break;
