@@ -48,7 +48,7 @@ static GstStaticPadTemplate gst_rtp_mpa_pay_src_template =
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"audio\", "
         "payload = (int) " GST_RTP_PAYLOAD_MPA_STRING ", "
-        "clock-rate = (int) 90000, " "encoding-name = (string) \"MPA\"; "
+        "clock-rate = (int) 90000; "
         "application/x-rtp, "
         "media = (string) \"audio\", "
         "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
@@ -174,7 +174,6 @@ gst_rtp_mpa_pay_flush (GstRtpMPAPay * rtpmpapay)
   while (avail > 0) {
     guint towrite;
     guint8 *payload;
-    guint8 *data;
     guint payload_len;
     guint packet_len;
 
@@ -207,8 +206,7 @@ gst_rtp_mpa_pay_flush (GstRtpMPAPay * rtpmpapay)
     payload[2] = frag_offset >> 8;
     payload[3] = frag_offset & 0xff;
 
-    data = (guint8 *) gst_adapter_peek (rtpmpapay->adapter, payload_len);
-    memcpy (&payload[4], data, payload_len);
+    gst_adapter_copy (rtpmpapay->adapter, &payload[4], 0, payload_len);
     gst_adapter_flush (rtpmpapay->adapter, payload_len);
 
     avail -= payload_len;
