@@ -25,6 +25,7 @@
 
 #include <gst/rtp/gstrtpbuffer.h>
 
+#include "fnv1hash.h"
 #include "gstrtptheorapay.h"
 
 #define THEORA_ID_LEN	42
@@ -49,8 +50,8 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
-        "payload = (int) [ 96, 127 ], "
-        "clock-rate = (int) [1, MAX ], " "encoding-name = (string) \"theora\""
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) 90000, " "encoding-name = (string) \"THEORA\""
         /* All required parameters
          *
          * "sampling = (string) { "YCbCr-4:2:0", "YCbCr-4:2:2", "YCbCr-4:4:4" } "
@@ -78,6 +79,8 @@ GST_BOILERPLATE (GstRtpTheoraPay, gst_rtp_theora_pay, GstBaseRTPPayload,
 
 static gboolean gst_rtp_theora_pay_setcaps (GstBaseRTPPayload * basepayload,
     GstCaps * caps);
+static GstStateChangeReturn gst_rtp_theora_pay_change_state (GstElement * element,
+    GstStateChange transition);
 static GstFlowReturn gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * pad,
     GstBuffer * buffer);
 
@@ -107,6 +110,8 @@ gst_rtp_theora_pay_class_init (GstRtpTheoraPayClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
+  gstelement_class->change_state = gst_rtp_theora_pay_change_state;
+
   gstbasertppayload_class->set_caps = gst_rtp_theora_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_theora_pay_handle_buffer;
 
@@ -118,6 +123,19 @@ static void
 gst_rtp_theora_pay_init (GstRtpTheoraPay * rtptheorapay,
     GstRtpTheoraPayClass * klass)
 {
+  /* needed because of GST_BOILERPLATE */
+}
+
+static void
+gst_rtp_theora_pay_cleanup (GstRtpTheoraPay *rtptheorapay)
+{
+  g_list_foreach (rtptheorapay->headers, (GFunc) gst_mini_object_unref, NULL);
+  g_list_free (rtptheorapay->headers);
+  rtptheorapay->headers = NULL;
+
+  if (rtptheorapay->packet)
+    gst_buffer_unref (rtptheorapay->packet);
+  rtptheorapay->packet = NULL;
 }
 
 static gboolean
@@ -162,6 +180,7 @@ gst_rtp_theora_pay_init_packet (GstRtpTheoraPay * rtptheorapay, guint8 TDT,
       gst_rtp_buffer_new_allocate_len (GST_BASE_RTP_PAYLOAD_MTU
       (rtptheorapay), 0, 0);
   gst_rtp_theora_pay_reset_packet (rtptheorapay, TDT);
+
   GST_BUFFER_TIMESTAMP (rtptheorapay->packet) = timestamp;
 }
 
@@ -202,6 +221,8 @@ gst_rtp_theora_pay_flush_packet (GstRtpTheoraPay * rtptheorapay)
   hlen = gst_rtp_buffer_calc_header_len (0);
   GST_BUFFER_SIZE (rtptheorapay->packet) = hlen + rtptheorapay->payload_pos;
 
+  GST_BUFFER_DURATION (rtptheorapay->packet) = rtptheorapay->payload_duration;
+
   /* push, this gives away our ref to the packet, so clear it. */
   ret =
       gst_basertppayload_push (GST_BASE_RTP_PAYLOAD (rtptheorapay),
@@ -211,28 +232,46 @@ gst_rtp_theora_pay_flush_packet (GstRtpTheoraPay * rtptheorapay)
   return ret;
 }
 
+static gchar *
+encode_base64 (const guint8 * in, guint size, guint * len)
+{
+  gchar *ret, *d;
+  static const gchar *v =
+      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
+  *len = ((size + 2) / 3) * 4;
+  d = ret = (gchar *) g_malloc (*len + 1);
+  for (; size; in += 3) {       /* process tuplets */
+    *d++ = v[in[0] >> 2];       /* byte 1: high 6 bits (1) */
+    /* byte 2: low 2 bits (1), high 4 bits (2) */
+    *d++ = v[((in[0] << 4) + (--size ? (in[1] >> 4) : 0)) & 0x3f];
+    /* byte 3: low 4 bits (2), high 2 bits (3) */
+    *d++ = size ? v[((in[1] << 2) + (--size ? (in[2] >> 6) : 0)) & 0x3f] : '=';
+    /* byte 4: low 6 bits (3) */
+    *d++ = size ? v[in[2] & 0x3f] : '=';
+    if (size)
+      size--;                   /* count third character if processed */
+  }
+  *d = '\0';                    /* tie off string */
+
+  return ret;                   /* return the resulting string */
+}
+
 static gboolean
 gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
 {
   GstRtpTheoraPay *rtptheorapay = GST_RTP_THEORA_PAY (basepayload);
   GList *walk;
-  guint length;
-  gchar *configuration;
-  gchar *wstr, *hstr;
-  guint8 *data;
+  guint length, size, n_headers, configlen;
+  gchar *wstr, *hstr, *configuration;
+  guint8 *data, *config;
   guint32 ident;
-  GValue v = { 0 };
-  GstBuffer *config;
 
   GST_DEBUG_OBJECT (rtptheorapay, "finish headers");
 
   if (!rtptheorapay->headers)
     goto no_headers;
 
-  /* we need exactly 2 header packets */
-  if (g_list_length (rtptheorapay->headers) != 2)
-    goto no_headers;
-
   /* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * |                     Number of packed headers                  |
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
@@ -251,31 +290,60 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
    *  0                   1                   2                   3
    *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   * |                   Ident                       |              ..
+   * |                   Ident                       | length       ..
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   * ..   length     |              Identification Header           ..
+   * ..              | n. of headers |    length1    |    length2   ..
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   * ..                    Identification Header                     |
+   * ..              |             Identification Header            ..
+   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   * .................................................................
+   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   * ..              |         Comment Header                       ..
+   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   * .................................................................
+   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   * ..                        Comment Header                        |
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * |                          Setup Header                        ..
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   * .................................................................
+   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * ..                         Setup Header                         |
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   *
    */
 
-  /* count the size of the headers first */
+  /* we need 4 bytes for the number of headers (which is always 1), 3 bytes for
+   * the ident, 2 bytes for length, 1 byte for n. of headers. */
+  size = 4 + 3 + 2 + 1;
+
+  /* count the size of the headers first and update the hash */
   length = 0;
+  n_headers = 0;
+  ident = fnv1_hash_32_new ();
   for (walk = rtptheorapay->headers; walk; walk = g_list_next (walk)) {
     GstBuffer *buf = GST_BUFFER_CAST (walk->data);
+    guint bsize;
 
-    length += GST_BUFFER_SIZE (buf);
+    bsize = GST_BUFFER_SIZE (buf);
+    length += bsize;
+    n_headers++;
+
+    /* count number of bytes needed for length fields, we don't need this for
+     * the last header. */
+    if (g_list_next (walk)) {
+      do {
+        size++;
+        bsize >>= 7;
+      } while (bsize);
+    }
+    /* update hash */
+    ident = fnv1_hash_32_update (ident, GST_BUFFER_DATA (buf),
+        GST_BUFFER_SIZE (buf));
   }
 
-  /* total config length is 4 bytes header number + size of the
-   *  headers + 2 bytes length + 3 bytes for the ident */
-  config = gst_buffer_new_and_alloc (length + 4 + 2 + 3);
-  data = GST_BUFFER_DATA (config);
+  /* packet length is header size + packet length */
+  configlen = size + length;
+  config = data = g_malloc (configlen);
 
   /* number of packed headers, we only pack 1 header */
   data[0] = 0;
@@ -283,20 +351,53 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
   data[2] = 0;
   data[3] = 1;
 
-  /* we generate a random ident for this configuration */
-  ident = rtptheorapay->payload_ident = g_random_int ();
+  ident = fnv1_hash_32_to_24 (ident);
+  rtptheorapay->payload_ident = ident;
+  GST_DEBUG_OBJECT (rtptheorapay, "ident 0x%08x", ident);
 
   /* take lower 3 bytes */
   data[4] = (ident >> 16) & 0xff;
   data[5] = (ident >> 8) & 0xff;
   data[6] = ident & 0xff;
 
-  /* store length minus the length of the fixed theora header */
-  data[7] = ((length - THEORA_ID_LEN) >> 8) & 0xff;
-  data[8] = (length - THEORA_ID_LEN) & 0xff;
+  /* store length of all theora headers */
+  data[7] = ((length) >> 8) & 0xff;
+  data[8] = (length) & 0xff;
+
+  /* store number of headers minus one. */
+  data[9] = n_headers - 1;
+  data += 10;
+
+  /* store length for each header */
+  for (walk = rtptheorapay->headers; walk; walk = g_list_next (walk)) {
+    GstBuffer *buf = GST_BUFFER_CAST (walk->data);
+    guint bsize, size, temp;
+
+    /* only need to store the length when it's not the last header */
+    if (!g_list_next (walk))
+      break;
+
+    bsize = GST_BUFFER_SIZE (buf);
+
+    /* calc size */
+    size = 0;
+    do {
+      size++;
+      bsize >>= 7;
+    } while (bsize);
+    temp = size;
+
+    bsize = GST_BUFFER_SIZE (buf);
+    /* write the size backwards */
+    while (size) {
+      size--;
+      data[size] = bsize & 0x7f;
+      bsize >>= 7;
+    }
+    data += temp;
+  }
 
   /* copy header data */
-  data += 9;
   for (walk = rtptheorapay->headers; walk; walk = g_list_next (walk)) {
     GstBuffer *buf = GST_BUFFER_CAST (walk->data);
 
@@ -304,15 +405,14 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
     data += GST_BUFFER_SIZE (buf);
   }
 
-  /* serialize buffer to base16 */
-  g_value_init (&v, GST_TYPE_BUFFER);
-  gst_value_take_buffer (&v, config);
-  configuration = gst_value_serialize (&v);
+  /* serialize to base64 */
+  configuration = encode_base64 (config, configlen, &size);
+  g_free (config);
 
   /* configure payloader settings */
   wstr = g_strdup_printf ("%d", rtptheorapay->width);
   hstr = g_strdup_printf ("%d", rtptheorapay->height);
-  gst_basertppayload_set_options (basepayload, "video", TRUE, "theora", 90000);
+  gst_basertppayload_set_options (basepayload, "video", TRUE, "THEORA", 90000);
   gst_basertppayload_set_outcaps (basepayload,
       "sampling", G_TYPE_STRING, "YCbCr-4:2:0",
       "width", G_TYPE_STRING, wstr,
@@ -325,7 +425,6 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
   g_free (wstr);
   g_free (hstr);
   g_free (configuration);
-  g_value_unset (&v);
 
   return TRUE;
 
@@ -427,7 +526,8 @@ gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   if (data[0] & 0x80) {
     /* header */
     if (data[0] == 0x80) {
-      /* identification, we need to parse this in order to get the clock rate. */
+      /* identification, we need to parse this in order to get the clock rate.
+       */
       if (G_UNLIKELY (!gst_rtp_theora_pay_parse_id (basepayload, data, size)))
         goto parse_id_failed;
       TDT = 1;
@@ -445,13 +545,8 @@ gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * basepayload,
 
   if (rtptheorapay->need_headers) {
     /* we need to collect the headers and construct a config string from them */
-    if (TDT == 2) {
-      GST_DEBUG_OBJECT (rtptheorapay,
-          "discard comment packet while collecting headers");
-      ret = GST_FLOW_OK;
-      goto done;
-    } else if (TDT != 0) {
-      GST_DEBUG_OBJECT (rtptheorapay, "collecting header");
+    if (TDT != 0) {
+      GST_DEBUG_OBJECT (rtptheorapay, "collecting header, buffer %p", buffer);
       /* append header to the list of headers */
       rtptheorapay->headers = g_list_append (rtptheorapay->headers, buffer);
       ret = GST_FLOW_OK;
@@ -545,6 +640,8 @@ gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * basepayload,
         rtptheorapay->payload_duration += duration;
     }
   }
+  gst_buffer_unref (buffer);
+
 done:
   return ret;
 
@@ -553,26 +650,55 @@ wrong_size:
   {
     GST_ELEMENT_WARNING (rtptheorapay, STREAM, DECODE,
         ("Invalid packet size (1 < %d <= 0xffff)", size), (NULL));
+    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 parse_id_failed:
   {
+    gst_buffer_unref (buffer);
     return GST_FLOW_ERROR;
   }
 unknown_header:
   {
     GST_ELEMENT_WARNING (rtptheorapay, STREAM, DECODE,
         (NULL), ("Ignoring unknown header received"));
+    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 header_error:
   {
     GST_ELEMENT_WARNING (rtptheorapay, STREAM, DECODE,
         (NULL), ("Error initializing header config"));
+    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 }
 
+static GstStateChangeReturn
+gst_rtp_theora_pay_change_state (GstElement * element, GstStateChange transition)
+{
+  GstRtpTheoraPay *rtptheorapay;
+  GstStateChangeReturn ret;
+
+  rtptheorapay = GST_RTP_THEORA_PAY (element);
+
+  switch (transition) {
+    default:
+      break;
+  }
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+
+  switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      gst_rtp_theora_pay_cleanup (rtptheorapay);
+      break;
+    default:
+      break;
+  }
+  return ret;
+}
+
 gboolean
 gst_rtp_theora_pay_plugin_init (GstPlugin * plugin)
 {
