@@ -25,13 +25,14 @@
 
 #include <gst/rtp/gstrtpbuffer.h>
 
+#include "fnv1hash.h"
 #include "gstrtpvorbispay.h"
 
 GST_DEBUG_CATEGORY_STATIC (rtpvorbispay_debug);
 #define GST_CAT_DEFAULT (rtpvorbispay_debug)
 
 /* references:
- * http://svn.xiph.org/trunk/vorbis/doc/draft-ietf-avt-rtp-vorbis-01.txt
+ * http://svn.xiph.org/trunk/vorbis/doc/draft-ietf-avt-rtp-vorbis-04.txt
  */
 
 /* elementfactory information */
@@ -47,8 +48,8 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"audio\", "
-        "payload = (int) [ 96, 127 ], "
-        "clock-rate = (int) [1, MAX ], " "encoding-name = (string) \"vorbis\""
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) [1, MAX ], " "encoding-name = (string) \"VORBIS\""
         /* All required parameters
          *
          * "encoding-params = (string) <num channels>"
@@ -74,6 +75,8 @@ GST_BOILERPLATE (GstRtpVorbisPay, gst_rtp_vorbis_pay, GstBaseRTPPayload,
 
 static gboolean gst_rtp_vorbis_pay_setcaps (GstBaseRTPPayload * basepayload,
     GstCaps * caps);
+static GstStateChangeReturn gst_rtp_vorbis_pay_change_state (GstElement * element,
+    GstStateChange transition);
 static GstFlowReturn gst_rtp_vorbis_pay_handle_buffer (GstBaseRTPPayload * pad,
     GstBuffer * buffer);
 
@@ -103,6 +106,8 @@ gst_rtp_vorbis_pay_class_init (GstRtpVorbisPayClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
+  gstelement_class->change_state = gst_rtp_vorbis_pay_change_state;
+
   gstbasertppayload_class->set_caps = gst_rtp_vorbis_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_vorbis_pay_handle_buffer;
 
@@ -114,6 +119,19 @@ static void
 gst_rtp_vorbis_pay_init (GstRtpVorbisPay * rtpvorbispay,
     GstRtpVorbisPayClass * klass)
 {
+  /* needed because of GST_BOILERPLATE */
+}
+
+static void
+gst_rtp_vorbis_pay_cleanup (GstRtpVorbisPay *rtpvorbispay)
+{
+  g_list_foreach (rtpvorbispay->headers, (GFunc) gst_mini_object_unref, NULL);
+  g_list_free (rtpvorbispay->headers);
+  rtpvorbispay->headers = NULL;
+
+  if (rtpvorbispay->packet)
+    gst_buffer_unref (rtpvorbispay->packet);
+  rtpvorbispay->packet = NULL;
 }
 
 static gboolean
@@ -133,7 +151,7 @@ gst_rtp_vorbis_pay_reset_packet (GstRtpVorbisPay * rtpvorbispay, guint8 VDT)
 {
   guint payload_len;
 
-  GST_DEBUG_OBJECT (rtpvorbispay, "reset packet");
+  GST_LOG_OBJECT (rtpvorbispay, "reset packet");
 
   rtpvorbispay->payload_pos = 4;
   payload_len = gst_rtp_buffer_get_payload_len (rtpvorbispay->packet);
@@ -148,7 +166,7 @@ static void
 gst_rtp_vorbis_pay_init_packet (GstRtpVorbisPay * rtpvorbispay, guint8 VDT,
     GstClockTime timestamp)
 {
-  GST_DEBUG_OBJECT (rtpvorbispay, "starting new packet, VDT: %d", VDT);
+  GST_LOG_OBJECT (rtpvorbispay, "starting new packet, VDT: %d", VDT);
 
   if (rtpvorbispay->packet)
     gst_buffer_unref (rtpvorbispay->packet);
@@ -172,7 +190,7 @@ gst_rtp_vorbis_pay_flush_packet (GstRtpVorbisPay * rtpvorbispay)
   if (!rtpvorbispay->packet || rtpvorbispay->payload_pos <= 4)
     return GST_FLOW_OK;
 
-  GST_DEBUG_OBJECT (rtpvorbispay, "flushing packet");
+  GST_LOG_OBJECT (rtpvorbispay, "flushing packet");
 
   /* fix header */
   payload = gst_rtp_buffer_get_payload (rtpvorbispay->packet);
@@ -198,6 +216,8 @@ gst_rtp_vorbis_pay_flush_packet (GstRtpVorbisPay * rtpvorbispay)
   hlen = gst_rtp_buffer_calc_header_len (0);
   GST_BUFFER_SIZE (rtpvorbispay->packet) = hlen + rtpvorbispay->payload_pos;
 
+  GST_BUFFER_DURATION (rtpvorbispay->packet) = rtpvorbispay->payload_duration;
+
   /* push, this gives away our ref to the packet, so clear it. */
   ret =
       gst_basertppayload_push (GST_BASE_RTP_PAYLOAD (rtpvorbispay),
@@ -207,27 +227,47 @@ gst_rtp_vorbis_pay_flush_packet (GstRtpVorbisPay * rtpvorbispay)
   return ret;
 }
 
+static gchar *
+encode_base64 (const guint8 * in, guint size, guint * len)
+{
+  gchar *ret, *d;
+  static const gchar v[] =
+      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
+  *len = ((size + 2) / 3) * 4;
+  d = ret = (gchar *) g_malloc (*len + 1);
+  for (; size; in += 3) {       /* process tuplets */
+    *d++ = v[in[0] >> 2];       /* byte 1: high 6 bits (1) */
+    /* byte 2: low 2 bits (1), high 4 bits (2) */
+    *d++ = v[((in[0] << 4) + (--size ? (in[1] >> 4) : 0)) & 0x3f];
+    /* byte 3: low 4 bits (2), high 2 bits (3) */
+    *d++ = size ? v[((in[1] << 2) + (--size ? (in[2] >> 6) : 0)) & 0x3f] : '=';
+    /* byte 4: low 6 bits (3) */
+    *d++ = size ? v[in[2] & 0x3f] : '=';
+    if (size)
+      size--;                   /* count third character if processed */
+  }
+  *d = '\0';                    /* tie off string */
+
+  return ret;                   /* return the resulting string */
+}
+
+
 static gboolean
 gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
 {
   GstRtpVorbisPay *rtpvorbispay = GST_RTP_VORBIS_PAY (basepayload);
   GList *walk;
-  guint length;
+  guint length, size, n_headers, configlen;
   gchar *cstr, *configuration;
-  guint8 *data;
+  guint8 *data, *config;
   guint32 ident;
-  GValue v = { 0 };
-  GstBuffer *config;
 
   GST_DEBUG_OBJECT (rtpvorbispay, "finish headers");
 
   if (!rtpvorbispay->headers)
     goto no_headers;
 
-  /* we need exactly 2 header packets */
-  if (g_list_length (rtpvorbispay->headers) != 2)
-    goto no_headers;
-
   /* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * |                     Number of packed headers                  |
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
@@ -246,31 +286,60 @@ gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
    *  0                   1                   2                   3
    *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   * |                   Ident                       |              ..
+   * |                   Ident                       | length       ..
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   * ..   length     |              Identification Header           ..
+   * ..              | n. of headers |    length1    |    length2   ..
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   * ..                    Identification Header                     |
+   * ..              |             Identification Header            ..
+   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   * .................................................................
+   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   * ..              |         Comment Header                       ..
+   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   * .................................................................
+   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   * ..                        Comment Header                        |
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * |                          Setup Header                        ..
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   * .................................................................
+   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * ..                         Setup Header                         |
    * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   *
    */
 
-  /* count the size of the headers first */
+  /* we need 4 bytes for the number of headers (which is always 1), 3 bytes for
+   * the ident, 2 bytes for length, 1 byte for n. of headers. */
+  size = 4 + 3 + 2 + 1;
+
+  /* count the size of the headers first and update the hash */
   length = 0;
+  n_headers = 0;
+  ident = fnv1_hash_32_new ();
   for (walk = rtpvorbispay->headers; walk; walk = g_list_next (walk)) {
     GstBuffer *buf = GST_BUFFER_CAST (walk->data);
+    guint bsize;
 
-    length += GST_BUFFER_SIZE (buf);
+    bsize = GST_BUFFER_SIZE (buf);
+    length += bsize;
+    n_headers++;
+
+    /* count number of bytes needed for length fields, we don't need this for
+     * the last header. */
+    if (g_list_next (walk)) {
+      do {
+        size++;
+        bsize >>= 7;
+      } while (bsize);
+    }
+    /* update hash */
+    ident = fnv1_hash_32_update (ident, GST_BUFFER_DATA (buf),
+        GST_BUFFER_SIZE (buf));
   }
 
-  /* total config length is 4 bytes header number + size of the
-   *  headers + 2 bytes length + 3 bytes for the ident */
-  config = gst_buffer_new_and_alloc (length + 4 + 2 + 3);
-  data = GST_BUFFER_DATA (config);
+  /* packet length is header size + packet length */
+  configlen = size + length;
+  config = data = g_malloc (configlen);
 
   /* number of packed headers, we only pack 1 header */
   data[0] = 0;
@@ -278,20 +347,53 @@ gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
   data[2] = 0;
   data[3] = 1;
 
-  /* we generate a random ident for this configuration */
-  ident = rtpvorbispay->payload_ident = g_random_int ();
+  ident = fnv1_hash_32_to_24 (ident);
+  rtpvorbispay->payload_ident = ident;
+  GST_DEBUG_OBJECT (rtpvorbispay, "ident 0x%08x", ident);
 
   /* take lower 3 bytes */
   data[4] = (ident >> 16) & 0xff;
   data[5] = (ident >> 8) & 0xff;
   data[6] = ident & 0xff;
 
-  /* store length minus the length of the fixed vorbis header */
-  data[7] = ((length - 30) >> 8) & 0xff;
-  data[8] = (length - 30) & 0xff;
+  /* store length of all vorbis headers */
+  data[7] = ((length) >> 8) & 0xff;
+  data[8] = (length) & 0xff;
+
+  /* store number of headers minus one. */
+  data[9] = n_headers - 1;
+  data += 10;
+
+  /* store length for each header */
+  for (walk = rtpvorbispay->headers; walk; walk = g_list_next (walk)) {
+    GstBuffer *buf = GST_BUFFER_CAST (walk->data);
+    guint bsize, size, temp;
+
+    /* only need to store the length when it's not the last header */
+    if (!g_list_next (walk))
+      break;
+
+    bsize = GST_BUFFER_SIZE (buf);
+
+    /* calc size */
+    size = 0;
+    do {
+      size++;
+      bsize >>= 7;
+    } while (bsize);
+    temp = size;
+
+    bsize = GST_BUFFER_SIZE (buf);
+    /* write the size backwards */
+    while (size) {
+      size--;
+      data[size] = bsize & 0x7f;
+      bsize >>= 7;
+    }
+    data += temp;
+  }
 
   /* copy header data */
-  data += 9;
   for (walk = rtpvorbispay->headers; walk; walk = g_list_next (walk)) {
     GstBuffer *buf = GST_BUFFER_CAST (walk->data);
 
@@ -299,14 +401,13 @@ gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
     data += GST_BUFFER_SIZE (buf);
   }
 
-  /* serialize buffer to base16 */
-  g_value_init (&v, GST_TYPE_BUFFER);
-  gst_value_take_buffer (&v, config);
-  configuration = gst_value_serialize (&v);
+  /* serialize to base64 */
+  configuration = encode_base64 (config, configlen, &size);
+  g_free (config);
 
   /* configure payloader settings */
   cstr = g_strdup_printf ("%d", rtpvorbispay->channels);
-  gst_basertppayload_set_options (basepayload, "audio", TRUE, "vorbis",
+  gst_basertppayload_set_options (basepayload, "audio", TRUE, "VORBIS",
       rtpvorbispay->rate);
   gst_basertppayload_set_outcaps (basepayload, "encoding-params", G_TYPE_STRING,
       cstr, "configuration", G_TYPE_STRING, configuration,
@@ -316,7 +417,6 @@ gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
       NULL);
   g_free (cstr);
   g_free (configuration);
-  g_value_unset (&v);
 
   return TRUE;
 
@@ -416,7 +516,7 @@ gst_rtp_vorbis_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   duration = GST_BUFFER_DURATION (buffer);
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
 
-  GST_DEBUG_OBJECT (rtpvorbispay, "size %u, duration %" GST_TIME_FORMAT,
+  GST_LOG_OBJECT (rtpvorbispay, "size %u, duration %" GST_TIME_FORMAT,
       size, GST_TIME_ARGS (duration));
 
   if (G_UNLIKELY (size < 1 || size > 0xffff))
@@ -444,12 +544,7 @@ gst_rtp_vorbis_pay_handle_buffer (GstBaseRTPPayload * basepayload,
 
   if (rtpvorbispay->need_headers) {
     /* we need to collect the headers and construct a config string from them */
-    if (VDT == 2) {
-      GST_DEBUG_OBJECT (rtpvorbispay,
-          "discard comment packet while collecting headers");
-      ret = GST_FLOW_OK;
-      goto done;
-    } else if (VDT != 0) {
+    if (VDT != 0) {
       GST_DEBUG_OBJECT (rtpvorbispay, "collecting header");
       /* append header to the list of headers */
       rtpvorbispay->headers = g_list_append (rtpvorbispay->headers, buffer);
@@ -496,7 +591,7 @@ gst_rtp_vorbis_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   while (size) {
     plen = MIN (rtpvorbispay->payload_left - 2, size);
 
-    GST_DEBUG_OBJECT (rtpvorbispay, "append %u bytes", plen);
+    GST_LOG_OBJECT (rtpvorbispay, "append %u bytes", plen);
 
     /* data is copied in the payload with a 2 byte length header */
     ppos[0] = (plen >> 8) & 0xff;
@@ -544,6 +639,8 @@ gst_rtp_vorbis_pay_handle_buffer (GstBaseRTPPayload * basepayload,
         rtpvorbispay->payload_duration += duration;
     }
   }
+  gst_buffer_unref (buffer);
+
 done:
   return ret;
 
@@ -552,26 +649,55 @@ wrong_size:
   {
     GST_ELEMENT_WARNING (rtpvorbispay, STREAM, DECODE,
         ("Invalid packet size (1 < %d <= 0xffff)", size), (NULL));
+    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 parse_id_failed:
   {
+    gst_buffer_unref (buffer);
     return GST_FLOW_ERROR;
   }
 unknown_header:
   {
     GST_ELEMENT_WARNING (rtpvorbispay, STREAM, DECODE,
         (NULL), ("Ignoring unknown header received"));
+    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 header_error:
   {
     GST_ELEMENT_WARNING (rtpvorbispay, STREAM, DECODE,
         (NULL), ("Error initializing header config"));
+    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 }
 
+static GstStateChangeReturn
+gst_rtp_vorbis_pay_change_state (GstElement * element, GstStateChange transition)
+{
+  GstRtpVorbisPay *rtpvorbispay;
+  GstStateChangeReturn ret;
+
+  rtpvorbispay = GST_RTP_VORBIS_PAY (element);
+
+  switch (transition) {
+    default:
+      break;
+  }
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+
+  switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      gst_rtp_vorbis_pay_cleanup (rtpvorbispay);
+      break;
+    default:
+      break;
+  }
+  return ret;
+}
+
 gboolean
 gst_rtp_vorbis_pay_plugin_init (GstPlugin * plugin)
 {
