@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2007> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -36,6 +36,14 @@
 #include "gstudp-marshal.h"
 #include "gstmultiudpsink.h"
 
+#include <stdio.h>
+#include <stdlib.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <errno.h>
+#include <string.h>
+
 GST_DEBUG_CATEGORY_STATIC (multiudpsink_debug);
 #define GST_CAT_DEFAULT (multiudpsink_debug)
 
@@ -49,7 +57,7 @@ static const GstElementDetails gst_multiudpsink_details =
 GST_ELEMENT_DETAILS ("UDP packet sender",
     "Sink/Network",
     "Send data over the network via UDP",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 /* MultiUDPSink signals and args */
 enum
@@ -68,15 +76,30 @@ enum
   LAST_SIGNAL
 };
 
+#define DEFAULT_SOCKFD             -1
+#define DEFAULT_CLOSEFD            TRUE
+#define DEFAULT_SOCK               -1
+#define DEFAULT_CLIENTS            NULL
+#define DEFAULT_AUTO_MULTICAST     TRUE
+
 enum
 {
   PROP_0,
   PROP_BYTES_TO_SERVE,
   PROP_BYTES_SERVED,
-
-  /* FILL ME */
+  PROP_SOCKFD,
+  PROP_CLOSEFD,
+  PROP_SOCK,
+  PROP_CLIENTS,
+  PROP_AUTO_MULTICAST
+      /* FILL ME */
 };
 
+#define CLOSE_IF_REQUESTED(udpctx)                                        \
+  if ((!udpctx->externalfd) || (udpctx->externalfd && udpctx->closefd))   \
+    CLOSE_SOCKET(udpctx->sock);                                           \
+  udpctx->sock = -1;
+
 static void gst_multiudpsink_base_init (gpointer g_class);
 static void gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass);
 static void gst_multiudpsink_init (GstMultiUDPSink * udpsink);
@@ -92,6 +115,13 @@ static void gst_multiudpsink_set_property (GObject * object, guint prop_id,
 static void gst_multiudpsink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
+static void gst_multiudpsink_add_internal (GstMultiUDPSink * sink,
+    const gchar * host, gint port, gboolean lock);
+static void gst_multiudpsink_clear_internal (GstMultiUDPSink * sink,
+    gboolean lock);
+
+static void free_client (GstUDPClient * client);
+
 static GstElementClass *parent_class = NULL;
 
 static guint gst_multiudpsink_signals[LAST_SIGNAL] = { 0 };
@@ -241,6 +271,27 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
       g_param_spec_uint64 ("bytes-served", "Bytes served",
           "Total number of bytes send to all clients", 0, G_MAXUINT64, 0,
           G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class, PROP_SOCKFD,
+      g_param_spec_int ("sockfd", "Socket Handle",
+          "Socket to use for UDP sending. (-1 == allocate)",
+          -1, G_MAXINT, DEFAULT_SOCKFD, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_CLOSEFD,
+      g_param_spec_boolean ("closefd", "Close sockfd",
+          "Close sockfd if passed as property on state change",
+          DEFAULT_CLOSEFD, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_SOCK,
+      g_param_spec_int ("sock", "Socket Handle",
+          "Socket currently in use for UDP sending. (-1 == no socket)",
+          -1, G_MAXINT, DEFAULT_SOCK, G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class, PROP_CLIENTS,
+      g_param_spec_string ("clients", "Clients",
+          "A comma separated list of host:port pairs with destinations",
+          DEFAULT_CLIENTS, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_AUTO_MULTICAST,
+      g_param_spec_boolean ("auto-multicast",
+          "Automatically join/leave multicast groups",
+          "Automatically join/leave the multicast groups, FALSE means user"
+          " has to do it himself", DEFAULT_AUTO_MULTICAST, G_PARAM_READWRITE));
 
   gstelement_class->change_state = gst_multiudpsink_change_state;
 
@@ -260,6 +311,11 @@ gst_multiudpsink_init (GstMultiUDPSink * sink)
   WSA_STARTUP (sink);
 
   sink->client_lock = g_mutex_new ();
+  sink->sock = -1;
+  sink->sockfd = DEFAULT_SOCKFD;
+  sink->closefd = DEFAULT_CLOSEFD;
+  sink->externalfd = (sink->sockfd != -1);
+  sink->auto_multicast = DEFAULT_AUTO_MULTICAST;
 }
 
 static void
@@ -269,6 +325,9 @@ gst_multiudpsink_finalize (GObject * object)
 
   sink = GST_MULTIUDPSINK (object);
 
+  g_list_foreach (sink->clients, (GFunc) free_client, NULL);
+  g_list_free (sink->clients);
+
   g_mutex_free (sink->client_lock);
 
   WSA_CLEANUP (object);
@@ -337,6 +396,61 @@ send_error:
   }
 }
 
+static void
+gst_multiudpsink_set_clients_string (GstMultiUDPSink * sink,
+    const gchar * string)
+{
+  gchar **clients;
+  gint i;
+
+  clients = g_strsplit (string, ",", 0);
+
+  g_mutex_lock (sink->client_lock);
+  /* clear all existing clients */
+  gst_multiudpsink_clear_internal (sink, FALSE);
+  for (i = 0; clients[i]; i++) {
+    gchar *host, *p;
+    gint port = 0;
+
+    host = clients[i];
+    p = strstr (clients[i], ":");
+    if (p != NULL) {
+      *p = '\0';
+      port = atoi (p + 1);
+    }
+    if (port != 0)
+      gst_multiudpsink_add_internal (sink, host, port, FALSE);
+  }
+  g_mutex_unlock (sink->client_lock);
+
+  g_strfreev (clients);
+}
+
+static gchar *
+gst_multiudpsink_get_clients_string (GstMultiUDPSink * sink)
+{
+  GString *str;
+  GList *clients;
+
+  str = g_string_new ("");
+
+  g_mutex_lock (sink->client_lock);
+  clients = sink->clients;
+  while (clients) {
+    GstUDPClient *client;
+
+    client = (GstUDPClient *) clients->data;
+
+    clients = g_list_next (clients);
+
+    g_string_append_printf (str, "%s:%d%s", client->host, client->port,
+        (clients ? "," : ""));
+  }
+  g_mutex_unlock (sink->client_lock);
+
+  return g_string_free (str, FALSE);
+}
+
 static void
 gst_multiudpsink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -346,6 +460,19 @@ gst_multiudpsink_set_property (GObject * object, guint prop_id,
   udpsink = GST_MULTIUDPSINK (object);
 
   switch (prop_id) {
+    case PROP_SOCKFD:
+      udpsink->sockfd = g_value_get_int (value);
+      GST_DEBUG_OBJECT (udpsink, "setting SOCKFD to %d", udpsink->sockfd);
+      break;
+    case PROP_CLOSEFD:
+      udpsink->closefd = g_value_get_boolean (value);
+      break;
+    case PROP_CLIENTS:
+      gst_multiudpsink_set_clients_string (udpsink, g_value_get_string (value));
+      break;
+    case PROP_AUTO_MULTICAST:
+      udpsink->auto_multicast = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -367,7 +494,22 @@ gst_multiudpsink_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_BYTES_SERVED:
       g_value_set_uint64 (value, udpsink->bytes_served);
       break;
-
+    case PROP_SOCKFD:
+      g_value_set_int (value, udpsink->sockfd);
+      break;
+    case PROP_CLOSEFD:
+      g_value_set_boolean (value, udpsink->closefd);
+      break;
+    case PROP_SOCK:
+      g_value_set_int (value, udpsink->sock);
+      break;
+    case PROP_CLIENTS:
+      g_value_take_string (value,
+          gst_multiudpsink_get_clients_string (udpsink));
+      break;
+    case PROP_AUTO_MULTICAST:
+      g_value_set_boolean (value, udpsink->auto_multicast);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -395,6 +537,15 @@ join_multicast (GstUDPClient * client)
     perror ("setsockopt IP_MULTICAST_LOOP\n");
 }
 
+static void
+leave_multicast (GstUDPClient * client)
+{
+  if (setsockopt (*(client->sock), IPPROTO_IP, IP_DROP_MEMBERSHIP,
+          &(client->multi_addr), sizeof (client->multi_addr)) < 0)
+    GST_WARNING ("setsockopt IP_DROP_MEMBERSHIP failed '%s'",
+        g_strerror (errno));
+}
+
 /* create a socket for sending to remote machine */
 static gboolean
 gst_multiudpsink_init_send (GstMultiUDPSink * sink)
@@ -404,9 +555,17 @@ gst_multiudpsink_init_send (GstMultiUDPSink * sink)
   GList *clients;
   GstUDPClient *client;
 
-  /* create sender socket */
-  if ((sink->sock = socket (AF_INET, SOCK_DGRAM, 0)) == -1)
-    goto no_socket;
+  if (sink->sockfd == -1) {
+    /* create sender socket */
+    if ((sink->sock = socket (AF_INET, SOCK_DGRAM, 0)) == -1)
+      goto no_socket;
+
+    sink->externalfd = FALSE;
+  } else {
+    /* we use the configured socket */
+    sink->sock = sink->sockfd;
+    sink->externalfd = TRUE;
+  }
 
   bc_val = 1;
   if ((ret =
@@ -421,10 +580,11 @@ gst_multiudpsink_init_send (GstMultiUDPSink * sink)
 
   for (clients = sink->clients; clients; clients = g_list_next (clients)) {
     client = (GstUDPClient *) clients->data;
-    if (client->multi_addr.imr_multiaddr.s_addr)
+    if (client->multi_addr.imr_multiaddr.s_addr && sink->auto_multicast)
       join_multicast (client);
   }
   return TRUE;
+
   /* ERRORS */
 no_socket:
   {
@@ -434,6 +594,7 @@ no_socket:
   }
 no_broadcast:
   {
+    CLOSE_IF_REQUESTED (sink);
     GST_ELEMENT_ERROR (sink, RESOURCE, SETTINGS, (NULL),
         ("Could not set broadcast socket option (%d): %s", errno,
             g_strerror (errno)));
@@ -444,11 +605,12 @@ no_broadcast:
 static void
 gst_multiudpsink_close (GstMultiUDPSink * sink)
 {
-  CLOSE_SOCKET (sink->sock);
+  CLOSE_IF_REQUESTED (sink);
 }
 
-void
-gst_multiudpsink_add (GstMultiUDPSink * sink, const gchar * host, gint port)
+static void
+gst_multiudpsink_add_internal (GstMultiUDPSink * sink, const gchar * host,
+    gint port, gboolean lock)
 {
   struct hostent *he;
   struct in_addr addr;
@@ -472,18 +634,20 @@ gst_multiudpsink_add (GstMultiUDPSink * sink, const gchar * host, gint port)
   /* if its an IP address */
   if (inet_aton (host, &addr)) {
     /* check if its a multicast address */
-    if ((ntohl (addr.s_addr) & 0xe0000000) == 0xe0000000) {
-      GST_DEBUG_OBJECT (sink, "multicast address  detected");
+    if ((ntohl (addr.s_addr) & 0xf0000000) == 0xe0000000) {
+      GST_DEBUG_OBJECT (sink, "multicast address detected");
       client->multi_addr.imr_multiaddr.s_addr = addr.s_addr;
       client->multi_addr.imr_interface.s_addr = INADDR_ANY;
 
       client->theiraddr.sin_addr = client->multi_addr.imr_multiaddr;
 
     } else {
+      GST_DEBUG_OBJECT (sink, "normal address detected");
       client->theiraddr.sin_addr = *((struct in_addr *) &addr);
     }
     /* if init_send has already been called, set sockopts for multicast */
-    if (*client->sock > 0)
+    if (*client->sock > 0 && client->multi_addr.imr_multiaddr.s_addr &&
+        sink->auto_multicast)
       join_multicast (client);
   }
   /* we dont need to lookup for localhost */
@@ -497,9 +661,11 @@ gst_multiudpsink_add (GstMultiUDPSink * sink, const gchar * host, gint port)
     goto host_error;
   }
 
-  g_mutex_lock (sink->client_lock);
+  if (lock)
+    g_mutex_lock (sink->client_lock);
   sink->clients = g_list_prepend (sink->clients, client);
-  g_mutex_unlock (sink->client_lock);
+  if (lock)
+    g_mutex_unlock (sink->client_lock);
 
   g_signal_emit (G_OBJECT (sink),
       gst_multiudpsink_signals[SIGNAL_CLIENT_ADDED], 0, host, port);
@@ -516,6 +682,12 @@ host_error:
   }
 }
 
+void
+gst_multiudpsink_add (GstMultiUDPSink * sink, const gchar * host, gint port)
+{
+  gst_multiudpsink_add_internal (sink, host, port, TRUE);
+}
+
 static gint
 client_compare (GstUDPClient * a, GstUDPClient * b)
 {
@@ -556,6 +728,10 @@ gst_multiudpsink_remove (GstMultiUDPSink * sink, const gchar * host, gint port)
   g_get_current_time (&now);
   client->disconnect_time = GST_TIMEVAL_TO_TIME (now);
 
+  if (*(client->sock) != -1 && client->multi_addr.imr_multiaddr.s_addr
+      && sink->auto_multicast)
+    leave_multicast (client);
+
   /* Unlock to emit signal before we delete the actual client */
   g_mutex_unlock (sink->client_lock);
   g_signal_emit (G_OBJECT (sink),
@@ -579,17 +755,25 @@ not_found:
   }
 }
 
-void
-gst_multiudpsink_clear (GstMultiUDPSink * sink)
+static void
+gst_multiudpsink_clear_internal (GstMultiUDPSink * sink, gboolean lock)
 {
   GST_DEBUG_OBJECT (sink, "clearing");
   /* we only need to remove the client structure, there is no additional
    * socket or anything to free for UDP */
-  g_mutex_lock (sink->client_lock);
+  if (lock)
+    g_mutex_lock (sink->client_lock);
   g_list_foreach (sink->clients, (GFunc) free_client, sink);
   g_list_free (sink->clients);
   sink->clients = NULL;
-  g_mutex_unlock (sink->client_lock);
+  if (lock)
+    g_mutex_unlock (sink->client_lock);
+}
+
+void
+gst_multiudpsink_clear (GstMultiUDPSink * sink)
+{
+  gst_multiudpsink_clear_internal (sink, TRUE);
 }
 
 GValueArray *
