@@ -71,16 +71,47 @@
  * default buffer size is typically 50K and can be increased to maximally 100K.
  * </para>
  * <para>
+ * The "skip-first-bytes" property is used to strip off an arbitrary number of
+ * bytes from the start of the raw udp packet and can be used to strip off
+ * proprietary header, for example. 
+ * </para>
+ * <para>
  * The udpsrc is always a live source. It does however not provide a GstClock, this
  * is left for upstream elements such as an RTP session manager or demuxer (such
- * as an MPEG demuxer).
+ * as an MPEG demuxer). As with all live sources, the captured buffers will have
+ * their timestamp set to the current running time of the pipeline.
  * </para>
  * <para>
  * udpsrc implements a GstURIHandler interface that handles udp://host:port type
  * URIs.
  * </para>
  * <para>
- * Last reviewed on 2006-07-27 (0.10.4)
+ * If the <link linkend="GstUDPSrc--timeout">timeout property</link> is set to a
+ * value bigger than 0, udpsrc will generate an element message named
+ * <classname>&quot;GstUDPSrcTimeout&quot;</classname>
+ * if no data was recieved in the given timeout.
+ * The message's structure contains one field:
+ * <itemizedlist>
+ * <listitem>
+ *   <para>
+ *   #guint64
+ *   <classname>&quot;timeout&quot;</classname>: the timeout in microseconds that
+ *   expired when waiting for data.
+ *   </para>
+ * </listitem>
+ * </itemizedlist>
+ * The message is typically used to detect that no UDP arrives in the receiver
+ * because it is blocked by a firewall.
+ * </para>
+ * <para>
+ * A custom file descriptor can be configured with the 
+ * <link linkend="GstUDPSrc--sockfd">sockfd property</link>. The socket will be
+ * closed when setting the element to READY by default. This behaviour can be
+ * overriden with the <link linkend="GstUDPSrc--closefd">closefd property</link>,
+ * in which case the application is responsible for closing the file descriptor.
+ * </para>
+ * <para>
+ * Last reviewed on 2007-09-20 (0.10.7)
  * </para>
  * </refsect2>
  */
@@ -89,9 +120,19 @@
 #endif
 
 #include "gstudpsrc.h"
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#include <stdlib.h>
+
+#if defined _MSC_VER && (_MSC_VER >= 1400)
+#include <io.h>
+#endif
 
 #include <gst/netbuffer/gstnetbuffer.h>
+#ifdef G_OS_WIN32
+typedef int socklen_t;
+#endif
 
 #ifdef HAVE_FIONREAD_IN_SYS_FILIO
 #include <sys/filio.h>
@@ -108,10 +149,10 @@ GST_DEBUG_CATEGORY_STATIC (udpsrc_debug);
 #define WRITE_SOCKET(src)      src->control_sock[1]
 #define READ_SOCKET(src)       src->control_sock[0]
 
-#define SEND_COMMAND(src, command)              \
-G_STMT_START {                                  \
-  unsigned char c; c = command;                 \
-  write (WRITE_SOCKET(src), &c, 1);             \
+#define SEND_COMMAND(src, command, res)          \
+G_STMT_START {                                   \
+  unsigned char c; c = command;                  \
+  res = write (WRITE_SOCKET(src), &c, 1);        \
 } G_STMT_END
 
 #define READ_COMMAND(src, command, res)         \
@@ -119,6 +160,11 @@ G_STMT_START {                                  \
   res = read(READ_SOCKET(src), &command, 1);    \
 } G_STMT_END
 
+#define CLOSE_IF_REQUESTED(udpctx)                                        \
+  if ((!udpctx->externalfd) || (udpctx->externalfd && udpctx->closefd))   \
+    CLOSE_SOCKET(udpctx->sock);                                           \
+  udpctx->sock = -1;
+
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -133,10 +179,14 @@ GST_ELEMENT_DETAILS ("UDP packet receiver",
 
 #define UDP_DEFAULT_PORT                4951
 #define UDP_DEFAULT_MULTICAST_GROUP     "0.0.0.0"
-#define UDP_DEFAULT_BUFFER_SIZE		0
 #define UDP_DEFAULT_URI                 "udp://"UDP_DEFAULT_MULTICAST_GROUP":"G_STRINGIFY(UDP_DEFAULT_PORT)
 #define UDP_DEFAULT_CAPS                NULL
 #define UDP_DEFAULT_SOCKFD              -1
+#define UDP_DEFAULT_BUFFER_SIZE		0
+#define UDP_DEFAULT_TIMEOUT             0
+#define UDP_DEFAULT_SKIP_FIRST_BYTES	0
+#define UDP_DEFAULT_CLOSEFD            TRUE
+#define UDP_DEFAULT_SOCK                -1
 
 enum
 {
@@ -146,7 +196,11 @@ enum
   PROP_URI,
   PROP_CAPS,
   PROP_SOCKFD,
-  PROP_BUFFER_SIZE
+  PROP_BUFFER_SIZE,
+  PROP_TIMEOUT,
+  PROP_SKIP_FIRST_BYTES,
+  PROP_CLOSEFD,
+  PROP_SOCK
 };
 
 static void gst_udpsrc_uri_handler_init (gpointer g_iface, gpointer iface_data);
@@ -156,6 +210,8 @@ static GstFlowReturn gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf);
 static gboolean gst_udpsrc_start (GstBaseSrc * bsrc);
 static gboolean gst_udpsrc_stop (GstBaseSrc * bsrc);
 static gboolean gst_udpsrc_unlock (GstBaseSrc * bsrc);
+static gboolean gst_udpsrc_unlock_stop (GstBaseSrc * bsrc);
+static void gst_udpsrc_finalize (GObject * object);
 
 static void gst_udpsrc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -203,6 +259,7 @@ gst_udpsrc_class_init (GstUDPSrcClass * klass)
 
   gobject_class->set_property = gst_udpsrc_set_property;
   gobject_class->get_property = gst_udpsrc_get_property;
+  gobject_class->finalize = gst_udpsrc_finalize;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PORT,
       g_param_spec_int ("port", "Port",
@@ -227,10 +284,27 @@ gst_udpsrc_class_init (GstUDPSrcClass * klass)
       g_param_spec_int ("buffer-size", "Buffer Size",
           "Size of the kernel receive buffer in bytes, 0=default", 0, G_MAXINT,
           UDP_DEFAULT_BUFFER_SIZE, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_TIMEOUT,
+      g_param_spec_uint64 ("timeout", "Timeout",
+          "Post a message after timeout microseconds (0 = disabled)", 0,
+          G_MAXUINT64, UDP_DEFAULT_TIMEOUT, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass),
+      PROP_SKIP_FIRST_BYTES, g_param_spec_int ("skip-first-bytes",
+          "Skip first bytes", "number of bytes to skip for each udp packet", 0,
+          G_MAXINT, UDP_DEFAULT_SKIP_FIRST_BYTES, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_CLOSEFD,
+      g_param_spec_boolean ("closefd", "Close sockfd",
+          "Close sockfd if passed as property on state change",
+          UDP_DEFAULT_CLOSEFD, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_SOCK,
+      g_param_spec_int ("sock", "Socket Handle",
+          "Socket currently in use for UDP reception. (-1 = no socket)",
+          -1, G_MAXINT, UDP_DEFAULT_SOCK, G_PARAM_READABLE));
 
   gstbasesrc_class->start = gst_udpsrc_start;
   gstbasesrc_class->stop = gst_udpsrc_stop;
   gstbasesrc_class->unlock = gst_udpsrc_unlock;
+  gstbasesrc_class->unlock_stop = gst_udpsrc_unlock_stop;
   gstbasesrc_class->get_caps = gst_udpsrc_getcaps;
 
   gstpushsrc_class->create = gst_udpsrc_create;
@@ -243,10 +317,35 @@ gst_udpsrc_init (GstUDPSrc * udpsrc, GstUDPSrcClass * g_class)
 
   gst_base_src_set_live (GST_BASE_SRC (udpsrc), TRUE);
   udpsrc->port = UDP_DEFAULT_PORT;
-  udpsrc->sock = UDP_DEFAULT_SOCKFD;
+  udpsrc->sockfd = UDP_DEFAULT_SOCKFD;
   udpsrc->multi_group = g_strdup (UDP_DEFAULT_MULTICAST_GROUP);
   udpsrc->uri = g_strdup (UDP_DEFAULT_URI);
   udpsrc->buffer_size = UDP_DEFAULT_BUFFER_SIZE;
+  udpsrc->timeout = UDP_DEFAULT_TIMEOUT;
+  udpsrc->skip_first_bytes = UDP_DEFAULT_SKIP_FIRST_BYTES;
+  udpsrc->closefd = UDP_DEFAULT_CLOSEFD;
+  udpsrc->externalfd = (udpsrc->sockfd != -1);
+
+  udpsrc->sock = UDP_DEFAULT_SOCK;
+  udpsrc->control_sock[0] = -1;
+  udpsrc->control_sock[1] = -1;
+  gst_base_src_set_format (GST_BASE_SRC (udpsrc), GST_FORMAT_TIME);
+  gst_base_src_set_do_timestamp (GST_BASE_SRC (udpsrc), TRUE);
+}
+
+static void
+gst_udpsrc_finalize (GObject * object)
+{
+  GstUDPSrc *udpsrc;
+
+  udpsrc = GST_UDPSRC (object);
+
+  if (udpsrc->caps)
+    gst_caps_unref (udpsrc->caps);
+  g_free (udpsrc->multi_group);
+  g_free (udpsrc->uri);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static GstCaps *
@@ -271,13 +370,12 @@ gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
   socklen_t len;
   fd_set read_fds;
   guint max_sock;
-  gchar *pktdata;
+  guint8 *pktdata;
   gint pktsize;
 
 #ifdef G_OS_UNIX
   gint readsize;
-#endif
-#ifdef G_OS_WIN32
+#elif defined G_OS_WIN32
   gulong readsize;
 #endif
   gint ret;
@@ -285,32 +383,52 @@ gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
 
   udpsrc = GST_UDPSRC (psrc);
 
-  FD_ZERO (&read_fds);
-  FD_SET (udpsrc->sock, &read_fds);
-#ifdef G_OS_UNIX
-  FD_SET (READ_SOCKET (udpsrc), &read_fds);
-#endif
-  max_sock = MAX (udpsrc->sock, READ_SOCKET (udpsrc));
+retry:
+  /* quick check, avoid going in select when we already have data */
+  readsize = 0;
+  if ((ret = IOCTL_SOCKET (udpsrc->sock, FIONREAD, &readsize)) < 0)
+    goto ioctl_failed;
+
+  if (readsize > 0)
+    goto no_select;
 
   do {
     gboolean stop;
+    struct timeval timeval, *timeout;
+
+    FD_ZERO (&read_fds);
+    FD_SET (udpsrc->sock, &read_fds);
+#ifndef G_OS_WIN32
+    FD_SET (READ_SOCKET (udpsrc), &read_fds);
+#endif
+    max_sock = MAX (udpsrc->sock, READ_SOCKET (udpsrc));
 
     try_again = FALSE;
     stop = FALSE;
 
-    GST_LOG_OBJECT (udpsrc, "doing select");
+    GST_LOG_OBJECT (udpsrc, "doing select, timeout %" G_GUINT64_FORMAT,
+        udpsrc->timeout);
+
+    if (udpsrc->timeout > 0) {
+      timeval.tv_sec = udpsrc->timeout / 1000000;
+      timeval.tv_usec = udpsrc->timeout % 1000000;
+      timeout = &timeval;
+    } else {
+      timeout = NULL;
+    }
+
 #ifdef G_OS_WIN32
     if (((max_sock + 1) != READ_SOCKET (udpsrc)) ||
         ((max_sock + 1) != WRITE_SOCKET (udpsrc))) {
-      ret = select (max_sock + 1, &read_fds, NULL, NULL, NULL);
+      ret = select (max_sock + 1, &read_fds, NULL, NULL, timeout);
     } else {
       ret = 1;
     }
 #else
-    ret = select (max_sock + 1, &read_fds, NULL, NULL, NULL);
+    ret = select (max_sock + 1, &read_fds, NULL, NULL, timeout);
 #endif
     GST_LOG_OBJECT (udpsrc, "select returned %d", ret);
-    if (ret <= 0) {
+    if (ret < 0) {
 #ifdef G_OS_WIN32
       if (WSAGetLastError () != WSAEINTR)
         goto select_error;
@@ -319,50 +437,41 @@ gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
         goto select_error;
 #endif
       try_again = TRUE;
+    } else if (ret == 0) {
+      /* timeout, post element message */
+      gst_element_post_message (GST_ELEMENT_CAST (udpsrc),
+          gst_message_new_element (GST_OBJECT_CAST (udpsrc),
+              gst_structure_new ("GstUDPSrcTimeout",
+                  "timeout", G_TYPE_UINT64, udpsrc->timeout, NULL)));
+      try_again = TRUE;
     } else {
-      if (FD_ISSET (READ_SOCKET (udpsrc), &read_fds)) {
-        /* got control message */
-        while (TRUE) {
-          gchar command;
-          int res;
-
-          READ_COMMAND (udpsrc, command, res);
-          if (res < 0) {
-            GST_LOG_OBJECT (udpsrc, "no more commands");
-            /* no more commands */
-            break;
-          }
-
-          switch (command) {
-            case CONTROL_STOP:
-              /* break out of the select loop */
-              GST_LOG_OBJECT (udpsrc, "stop");
-              /* stop this function */
-              stop = TRUE;
-              break;
-            default:
-              GST_WARNING_OBJECT (udpsrc, "unkown");
-              g_warning ("multiudpsink: unknown control message received");
-              break;
-          }
-        }
-      }
+      if (FD_ISSET (READ_SOCKET (udpsrc), &read_fds))
+        goto stopped;
     }
-    if (stop)
-      goto stopped;
   } while (try_again);
 
-  /* ask how much is available for reading on the socket */
+  /* ask how much is available for reading on the socket, this should be exactly
+   * one UDP packet. We will check the return value, though, because in some
+   * case it can return 0 and we don't want a 0 sized buffer. */
+  readsize = 0;
   if ((ret = IOCTL_SOCKET (udpsrc->sock, FIONREAD, &readsize)) < 0)
     goto ioctl_failed;
 
-  GST_LOG_OBJECT (udpsrc, "ioctl says %d bytes available", readsize);
+  /* if we get here and there is nothing to read from the socket, the select got
+   * woken up by activity on the socket but it was not a read. We how someone
+   * will also do something with the socket so that we don't go into an infinite
+   * loop in the select(). */
+  if (!readsize)
+    goto retry;
+
+no_select:
+  GST_LOG_OBJECT (udpsrc, "ioctl says %d bytes available", (int) readsize);
 
   pktdata = g_malloc (readsize);
   pktsize = readsize;
 
-  len = sizeof (struct sockaddr);
   while (TRUE) {
+    len = sizeof (struct sockaddr);
     ret = recvfrom (udpsrc->sock, pktdata, pktsize,
         0, (struct sockaddr *) &tmpaddr, &len);
     if (ret < 0) {
@@ -374,8 +483,17 @@ gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
 
   /* special case buffer so receivers can also track the address */
   outbuf = gst_netbuffer_new ();
-  GST_BUFFER_DATA (outbuf) = (guint8 *) pktdata;
-  GST_BUFFER_MALLOCDATA (outbuf) = (guint8 *) pktdata;
+  GST_BUFFER_MALLOCDATA (outbuf) = pktdata;
+
+  /* patch pktdata and len when stripping off the headers */
+  if (udpsrc->skip_first_bytes != 0) {
+    if (G_UNLIKELY (readsize <= udpsrc->skip_first_bytes))
+      goto skip_error;
+
+    pktdata += udpsrc->skip_first_bytes;
+    ret -= udpsrc->skip_first_bytes;
+  }
+  GST_BUFFER_DATA (outbuf) = pktdata;
   GST_BUFFER_SIZE (outbuf) = ret;
 
   gst_netaddress_set_ip4_address (&outbuf->from, tmpaddr.sin_addr.s_addr,
@@ -383,7 +501,7 @@ gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
 
   gst_buffer_set_caps (GST_BUFFER_CAST (outbuf), udpsrc->caps);
 
-  GST_LOG_OBJECT (udpsrc, "read %d bytes", readsize);
+  GST_LOG_OBJECT (udpsrc, "read %d bytes", (int) readsize);
 
   *buf = GST_BUFFER_CAST (outbuf);
 
@@ -414,6 +532,12 @@ receive_error:
         ("receive error %d: %s (%d)", ret, g_strerror (errno), errno));
     return GST_FLOW_ERROR;
   }
+skip_error:
+  {
+    GST_ELEMENT_ERROR (udpsrc, STREAM, DECODE, (NULL),
+        ("UDP buffer to small to skip header"));
+    return GST_FLOW_ERROR;
+  }
 }
 
 /* Call this function when multicastgroup and/or port are updated */
@@ -423,6 +547,8 @@ gst_udpsrc_update_uri (GstUDPSrc * src)
 {
   g_free (src->uri);
   src->uri = g_strdup_printf ("udp://%s:%d", src->multi_group, src->port);
+
+  GST_DEBUG_OBJECT (src, "updated uri to %s", src->uri);
 }
 
 static gboolean
@@ -456,11 +582,12 @@ gst_udpsrc_set_uri (GstUDPSrc * src, const gchar * uri)
 
   return TRUE;
 
+  /* ERRORS */
 wrong_protocol:
   {
     g_free (protocol);
     GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
-        ("error parsing uri %s: wrong protocol", uri));
+        ("error parsing uri %s: wrong protocol (%s != udp)", uri, protocol));
     return FALSE;
   }
 }
@@ -485,7 +612,7 @@ gst_udpsrc_set_property (GObject * object, guint prop_id, const GValue * value,
       if (g_value_get_string (value) == NULL)
         udpsrc->multi_group = g_strdup (UDP_DEFAULT_MULTICAST_GROUP);
       else
-        udpsrc->multi_group = g_strdup (g_value_get_string (value));
+        udpsrc->multi_group = g_value_dup_string (value);
       gst_udpsrc_update_uri (udpsrc);
       break;
     case PROP_URI:
@@ -511,8 +638,17 @@ gst_udpsrc_set_property (GObject * object, guint prop_id, const GValue * value,
       break;
     }
     case PROP_SOCKFD:
-      udpsrc->sock = g_value_get_int (value);
-      GST_DEBUG ("setting SOCKFD to %d", udpsrc->sock);
+      udpsrc->sockfd = g_value_get_int (value);
+      GST_DEBUG ("setting SOCKFD to %d", udpsrc->sockfd);
+      break;
+    case PROP_TIMEOUT:
+      udpsrc->timeout = g_value_get_uint64 (value);
+      break;
+    case PROP_SKIP_FIRST_BYTES:
+      udpsrc->skip_first_bytes = g_value_get_int (value);
+      break;
+    case PROP_CLOSEFD:
+      udpsrc->closefd = g_value_get_boolean (value);
       break;
     default:
       break;
@@ -542,6 +678,18 @@ gst_udpsrc_get_property (GObject * object, guint prop_id, GValue * value,
       gst_value_set_caps (value, udpsrc->caps);
       break;
     case PROP_SOCKFD:
+      g_value_set_int (value, udpsrc->sockfd);
+      break;
+    case PROP_TIMEOUT:
+      g_value_set_uint64 (value, udpsrc->timeout);
+      break;
+    case PROP_SKIP_FIRST_BYTES:
+      g_value_set_int (value, udpsrc->skip_first_bytes);
+      break;
+    case PROP_CLOSEFD:
+      g_value_set_boolean (value, udpsrc->closefd);
+      break;
+    case PROP_SOCK:
       g_value_set_int (value, udpsrc->sock);
       break;
     default:
@@ -569,8 +717,8 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
   GST_DEBUG_OBJECT (src, "creating pipe");
 
   /* This should work on UNIX too. PF_UNIX sockets replaced with pipe */
-  /* pipe( CONTROL_SOCKETS(src) ) */
-  if ((ret = pipe (CONTROL_SOCKETS (src))) < 0)
+  /* pipe( CONTROL_SOCKETS(src), 4096, _O_BINARY ) */
+  if ((ret = _pipe (CONTROL_SOCKETS (src), 4096, _O_BINARY)) < 0)
     goto no_socket_pair;
 #else
   GST_DEBUG_OBJECT (src, "creating socket pair");
@@ -581,11 +729,16 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
   fcntl (WRITE_SOCKET (src), F_SETFL, O_NONBLOCK);
 #endif
 
-  if (src->sock == -1) {
-    if ((ret = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
+  if (!inet_aton (src->multi_group, &(src->multi_addr.imr_multiaddr)))
+    src->multi_addr.imr_multiaddr.s_addr = 0;
+
+  if (src->sockfd == -1) {
+    /* need to allocate a socket */
+    if ((ret = socket (PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
       goto no_socket;
 
     src->sock = ret;
+    src->externalfd = FALSE;
 
     reuse = 1;
     if ((ret =
@@ -596,23 +749,28 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
     memset (&src->myaddr, 0, sizeof (src->myaddr));
     src->myaddr.sin_family = AF_INET;   /* host byte order */
     src->myaddr.sin_port = htons (src->port);   /* short, network byte order */
-    src->myaddr.sin_addr.s_addr = INADDR_ANY;
+
+    if (src->multi_addr.imr_multiaddr.s_addr)
+      src->myaddr.sin_addr.s_addr = src->multi_addr.imr_multiaddr.s_addr;
+    else
+      src->myaddr.sin_addr.s_addr = INADDR_ANY;
 
     GST_DEBUG_OBJECT (src, "binding on port %d", src->port);
-    if ((ret =
-            bind (src->sock, (struct sockaddr *) &src->myaddr,
+    if ((ret = bind (src->sock, (struct sockaddr *) &src->myaddr,
                 sizeof (src->myaddr))) < 0)
       goto bind_error;
+  } else {
+    /* we use the configured socket */
+    src->sock = src->sockfd;
+    src->externalfd = TRUE;
   }
 
-  if (inet_aton (src->multi_group, &(src->multi_addr.imr_multiaddr))) {
-    if (src->multi_addr.imr_multiaddr.s_addr) {
-      src->multi_addr.imr_interface.s_addr = INADDR_ANY;
-      if ((ret =
-              setsockopt (src->sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
-                  &src->multi_addr, sizeof (src->multi_addr))) < 0)
-        goto membership;
-    }
+  if (src->multi_addr.imr_multiaddr.s_addr) {
+    src->multi_addr.imr_interface.s_addr = INADDR_ANY;
+    if ((ret =
+            setsockopt (src->sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+                &src->multi_addr, sizeof (src->multi_addr))) < 0)
+      goto membership;
   }
 
   len = sizeof (my_addr);
@@ -642,8 +800,7 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
     GST_DEBUG_OBJECT (src, "could not get udp buffer size");
 
   bc_val = 1;
-  if ((ret =
-          setsockopt (src->sock, SOL_SOCKET, SO_BROADCAST, &bc_val,
+  if ((ret = setsockopt (src->sock, SOL_SOCKET, SO_BROADCAST, &bc_val,
               sizeof (bc_val))) < 0)
     goto no_broadcast;
 
@@ -674,40 +831,35 @@ no_socket:
   }
 setsockopt_error:
   {
-    CLOSE_SOCKET (src->sock);
-    src->sock = -1;
+    CLOSE_IF_REQUESTED (src);
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("setsockopt failed %d: %s (%d)", ret, g_strerror (errno), errno));
     return FALSE;
   }
 bind_error:
   {
-    CLOSE_SOCKET (src->sock);
-    src->sock = -1;
+    CLOSE_IF_REQUESTED (src);
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("bind failed %d: %s (%d)", ret, g_strerror (errno), errno));
     return FALSE;
   }
 membership:
   {
-    CLOSE_SOCKET (src->sock);
-    src->sock = -1;
+    CLOSE_IF_REQUESTED (src);
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("could add membership %d: %s (%d)", ret, g_strerror (errno), errno));
     return FALSE;
   }
 getsockname_error:
   {
-    CLOSE_SOCKET (src->sock);
-    src->sock = -1;
+    CLOSE_IF_REQUESTED (src);
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("getsockname failed %d: %s (%d)", ret, g_strerror (errno), errno));
     return FALSE;
   }
 udpbuffer_error:
   {
-    CLOSE_SOCKET (src->sock);
-    src->sock = -1;
+    CLOSE_IF_REQUESTED (src);
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("Could not create a buffer of the size requested, %d: %s (%d)", ret,
             g_strerror (errno), errno));
@@ -715,8 +867,7 @@ udpbuffer_error:
   }
 no_broadcast:
   {
-    CLOSE_SOCKET (src->sock);
-    src->sock = -1;
+    CLOSE_IF_REQUESTED (src);
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("could not configure socket for broadcast %d: %s (%d)", ret,
             g_strerror (errno), errno));
@@ -728,11 +879,39 @@ static gboolean
 gst_udpsrc_unlock (GstBaseSrc * bsrc)
 {
   GstUDPSrc *src;
+  gint res;
 
   src = GST_UDPSRC (bsrc);
 
-  GST_DEBUG ("sending stop command");
-  SEND_COMMAND (src, CONTROL_STOP);
+  GST_LOG_OBJECT (src, "sending stop command");
+  SEND_COMMAND (src, CONTROL_STOP, res);
+  GST_LOG_OBJECT (src, "sent stop command %d", res);
+
+  return TRUE;
+}
+
+static gboolean
+gst_udpsrc_unlock_stop (GstBaseSrc * bsrc)
+{
+  GstUDPSrc *src;
+
+  src = GST_UDPSRC (bsrc);
+
+  GST_LOG_OBJECT (src, "clearing unlock command queue");
+
+  while (TRUE) {
+    gchar command;
+    int res;
+
+    GST_LOG_OBJECT (src, "reading command");
+
+    READ_COMMAND (src, command, res);
+    if (res <= 0) {
+      GST_LOG_OBJECT (src, "no more commands");
+      /* no more commands */
+      break;
+    }
+  }
 
   return TRUE;
 }
@@ -744,9 +923,10 @@ gst_udpsrc_stop (GstBaseSrc * bsrc)
 
   src = GST_UDPSRC (bsrc);
 
+  GST_DEBUG ("stopping, closing sockets");
+
   if (src->sock != -1) {
-    CLOSE_SOCKET (src->sock);
-    src->sock = -1;
+    CLOSE_IF_REQUESTED (src);
   }
 
   /* pipes on WIN32 else sockets */
@@ -766,7 +946,7 @@ gst_udpsrc_stop (GstBaseSrc * bsrc)
 
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
-static guint
+static GstURIType
 gst_udpsrc_uri_get_type (void)
 {
   return GST_URI_SRC;
