@@ -51,37 +51,16 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstcollectpads.h>
+#include <gst/controller/gstcontroller.h>
 
 #include <stdlib.h>
 #include <string.h>
 
+#include "videomixer.h"
+
 GST_DEBUG_CATEGORY_STATIC (gst_videomixer_debug);
 #define GST_CAT_DEFAULT gst_videomixer_debug
 
-#define GST_TYPE_VIDEO_MIXER_PAD (gst_videomixer_pad_get_type())
-#define GST_VIDEO_MIXER_PAD(obj) \
-        (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_VIDEO_MIXER_PAD, GstVideoMixerPad))
-#define GST_VIDEO_MIXER_PAD_CLASS(klass) \
-        (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VIDEO_MIXER_PAD, GstVideoMixerPadiClass))
-#define GST_IS_VIDEO_MIXER_PAD(obj) \
-        (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VIDEO_MIXER_PAD))
-#define GST_IS_VIDEO_MIXER_PAD_CLASS(klass) \
-        (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VIDEO_MIXER_PAD))
-
-typedef struct _GstVideoMixerPad GstVideoMixerPad;
-typedef struct _GstVideoMixerPadClass GstVideoMixerPadClass;
-typedef struct _GstVideoMixerCollect GstVideoMixerCollect;
-
-#define GST_TYPE_VIDEO_MIXER (gst_videomixer_get_type())
-#define GST_VIDEO_MIXER(obj) \
-        (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_VIDEO_MIXER, GstVideoMixer))
-#define GST_VIDEO_MIXER_CLASS(klass) \
-        (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VIDEO_MIXER, GstVideoMixerClass))
-#define GST_IS_VIDEO_MIXER(obj) \
-        (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VIDEO_MIXER))
-#define GST_IS_VIDEO_MIXER_CLASS(klass) \
-        (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VIDEO_MIXER))
-
 #define GST_VIDEO_MIXER_GET_STATE_LOCK(mix) \
   (GST_VIDEO_MIXER(mix)->state_lock)
 #define GST_VIDEO_MIXER_STATE_LOCK(mix) \
@@ -91,68 +70,9 @@ typedef struct _GstVideoMixerCollect GstVideoMixerCollect;
 
 static GType gst_videomixer_get_type (void);
 
-typedef struct _GstVideoMixer GstVideoMixer;
-typedef struct _GstVideoMixerClass GstVideoMixerClass;
 
-/**
- * GstVideoMixerBackground:
- * @VIDEO_MIXER_BACKGROUND_CHECKER: checker pattern background
- * @VIDEO_MIXER_BACKGROUND_BLACK: solid color black background
- * @VIDEO_MIXER_BACKGROUND_WHITE: solid color white background
- *
- * The different backgrounds videomixer can blend over.
- */
-typedef enum
-{
-  VIDEO_MIXER_BACKGROUND_CHECKER,
-  VIDEO_MIXER_BACKGROUND_BLACK,
-  VIDEO_MIXER_BACKGROUND_WHITE
-}
-GstVideoMixerBackground;
-
-/**
- * GstVideoMixer:
- *
- * The opaque #GstVideoMixer structure.
- */
-struct _GstVideoMixer
-{
-  GstElement element;
-
-  /* pad */
-  GstPad *srcpad;
-
-  /* Lock to prevent the state to change while blending */
-  GMutex *state_lock;
-  /* Sink pads using Collect Pads from core's base library */
-  GstCollectPads *collect;
-  /* sinkpads, a GSList of GstVideoMixerPads */
-  GSList *sinkpads;
-
-  gint numpads;
-
-  /* the master pad */
-  GstVideoMixerPad *master;
-
-  gint in_width, in_height;
-  gint out_width, out_height;
-  gboolean setcaps;
-
-  GstVideoMixerBackground background;
-
-  gint fps_n;
-  gint fps_d;
-};
-
-struct _GstVideoMixerClass
-{
-  GstElementClass parent_class;
-};
-
-static void gst_videomixer_pad_base_init (gpointer g_class);
 static void gst_videomixer_pad_class_init (GstVideoMixerPadClass * klass);
 static void gst_videomixer_pad_init (GstVideoMixerPad * mixerpad);
-
 static void gst_videomixer_pad_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void gst_videomixer_pad_set_property (GObject * object, guint prop_id,
@@ -173,39 +93,6 @@ enum
   ARG_PAD_ALPHA
 };
 
-struct _GstVideoMixerCollect
-{
-  GstCollectData collect;       /* we extend the CollectData */
-
-  GstBuffer *buffer;            /* the queued buffer for this pad */
-
-  GstVideoMixerPad *mixpad;
-};
-
-/* all information needed for one video stream */
-struct _GstVideoMixerPad
-{
-  GstPad parent;                /* subclass the pad */
-
-  gint64 queued;
-
-  guint in_width, in_height;
-  gint fps_n;
-  gint fps_d;
-
-  gint xpos, ypos;
-  guint zorder;
-  gint blend_mode;
-  gdouble alpha;
-
-  GstVideoMixerCollect *mixcol;
-};
-
-struct _GstVideoMixerPadClass
-{
-  GstPadClass parent_class;
-};
-
 static GType
 gst_videomixer_pad_get_type (void)
 {
@@ -214,7 +101,7 @@ gst_videomixer_pad_get_type (void)
   if (!videomixer_pad_type) {
     static const GTypeInfo videomixer_pad_info = {
       sizeof (GstVideoMixerPadClass),
-      gst_videomixer_pad_base_init,
+      NULL,
       NULL,
       (GClassInitFunc) gst_videomixer_pad_class_init,
       NULL,
@@ -230,11 +117,6 @@ gst_videomixer_pad_get_type (void)
   return videomixer_pad_type;
 }
 
-static void
-gst_videomixer_pad_base_init (gpointer g_class)
-{
-}
-
 static void
 gst_videomixer_pad_class_init (GstVideoMixerPadClass * klass)
 {
@@ -249,16 +131,19 @@ gst_videomixer_pad_class_init (GstVideoMixerPadClass * klass)
 
   g_object_class_install_property (gobject_class, ARG_PAD_ZORDER,
       g_param_spec_uint ("zorder", "Z-Order", "Z Order of the picture",
-          0, 10000, DEFAULT_PAD_ZORDER, G_PARAM_READWRITE));
+          0, 10000, DEFAULT_PAD_ZORDER,
+          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
   g_object_class_install_property (gobject_class, ARG_PAD_XPOS,
       g_param_spec_int ("xpos", "X Position", "X Position of the picture",
-          G_MININT, G_MAXINT, DEFAULT_PAD_XPOS, G_PARAM_READWRITE));
+          G_MININT, G_MAXINT, DEFAULT_PAD_XPOS,
+          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
   g_object_class_install_property (gobject_class, ARG_PAD_YPOS,
       g_param_spec_int ("ypos", "Y Position", "Y Position of the picture",
-          G_MININT, G_MAXINT, DEFAULT_PAD_YPOS, G_PARAM_READWRITE));
+          G_MININT, G_MAXINT, DEFAULT_PAD_YPOS,
+          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
   g_object_class_install_property (gobject_class, ARG_PAD_ALPHA,
-      g_param_spec_double ("alpha", "Alpha", "Alpha of the picture",
-          0.0, 1.0, DEFAULT_PAD_ALPHA, G_PARAM_READWRITE));
+      g_param_spec_double ("alpha", "Alpha", "Alpha of the picture", 0.0, 1.0,
+          DEFAULT_PAD_ALPHA, G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
 }
 
 static void
@@ -354,6 +239,7 @@ gst_videomixer_set_master_geometry (GstVideoMixer * mix)
       || mix->in_height != height || mix->fps_n != fps_n
       || mix->fps_d != fps_d) {
     mix->setcaps = TRUE;
+    mix->sendseg = TRUE;
     mix->master = master;
     mix->in_width = width;
     mix->in_height = height;
@@ -495,9 +381,6 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink_%d",
         "height = (int) [ 1, max ]," "framerate = (fraction) [ 0/1, MAX ]")
     );
 
-static void gst_videomixer_base_init (gpointer g_class);
-static void gst_videomixer_class_init (GstVideoMixerClass * klass);
-static void gst_videomixer_init (GstVideoMixer * videomixer);
 static void gst_videomixer_finalize (GObject * object);
 
 static GstCaps *gst_videomixer_getcaps (GstPad * pad);
@@ -514,33 +397,63 @@ static void gst_videomixer_get_property (GObject * object, guint prop_id,
 static GstStateChangeReturn gst_videomixer_change_state (GstElement * element,
     GstStateChange transition);
 
-static GstElementClass *parent_class = NULL;
-
 /*static guint gst_videomixer_signals[LAST_SIGNAL] = { 0 }; */
 
-static GType
-gst_videomixer_get_type (void)
+static void gst_videomixer_child_proxy_init (gpointer g_iface,
+    gpointer iface_data);
+static void _do_init (GType object_type);
+
+GST_BOILERPLATE_FULL (GstVideoMixer, gst_videomixer, GstElement,
+    GST_TYPE_ELEMENT, _do_init);
+
+static void
+_do_init (GType object_type)
 {
-  static GType videomixer_type = 0;
+  const GInterfaceInfo child_proxy_info = {
+    (GInterfaceInitFunc) gst_videomixer_child_proxy_init,
+    NULL,
+    NULL
+  };
+  g_type_add_interface_static (object_type, GST_TYPE_CHILD_PROXY,
+      &child_proxy_info);
+  GST_INFO ("GstChildProxy interface registered");
+}
 
-  if (!videomixer_type) {
-    static const GTypeInfo videomixer_info = {
-      sizeof (GstVideoMixerClass),
-      gst_videomixer_base_init,
-      NULL,
-      (GClassInitFunc) gst_videomixer_class_init,
-      NULL,
-      NULL,
-      sizeof (GstVideoMixer),
-      0,
-      (GInstanceInitFunc) gst_videomixer_init,
-    };
+static GstObject *
+gst_videomixer_child_proxy_get_child_by_index (GstChildProxy * child_proxy,
+    guint index)
+{
+  GstVideoMixer *mix = GST_VIDEO_MIXER (child_proxy);
+  GstObject *obj;
 
-    videomixer_type =
-        g_type_register_static (GST_TYPE_ELEMENT, "GstVideoMixer",
-        &videomixer_info, 0);
-  }
-  return videomixer_type;
+  GST_VIDEO_MIXER_STATE_LOCK (mix);
+  if ((obj = g_slist_nth_data (mix->sinkpads, index)))
+    gst_object_ref (obj);
+  GST_VIDEO_MIXER_STATE_UNLOCK (mix);
+  return obj;
+}
+
+static guint
+gst_videomixer_child_proxy_get_children_count (GstChildProxy * child_proxy)
+{
+  guint count = 0;
+  GstVideoMixer *mix = GST_VIDEO_MIXER (child_proxy);
+
+  GST_VIDEO_MIXER_STATE_LOCK (mix);
+  count = mix->numpads;
+  GST_VIDEO_MIXER_STATE_UNLOCK (mix);
+  GST_INFO ("Children Count: %d", count);
+  return count;
+}
+
+static void
+gst_videomixer_child_proxy_init (gpointer g_iface, gpointer iface_data)
+{
+  GstChildProxyInterface *iface = g_iface;
+
+  GST_INFO ("intializing child proxy interface");
+  iface->get_child_by_index = gst_videomixer_child_proxy_get_child_by_index;
+  iface->get_children_count = gst_videomixer_child_proxy_get_children_count;
 }
 
 static void
@@ -605,6 +518,7 @@ gst_videomixer_reset (GstVideoMixer * mix)
   mix->out_height = 0;
   mix->fps_n = mix->fps_d = 0;
   mix->setcaps = FALSE;
+  mix->sendseg = FALSE;
 
   /* clean up collect data */
   walk = mix->collect->data;
@@ -617,7 +531,7 @@ gst_videomixer_reset (GstVideoMixer * mix)
 }
 
 static void
-gst_videomixer_init (GstVideoMixer * mix)
+gst_videomixer_init (GstVideoMixer * mix, GstVideoMixerClass * g_class)
 {
   GstElementClass *klass = GST_ELEMENT_GET_CLASS (mix);
 
@@ -638,7 +552,6 @@ gst_videomixer_init (GstVideoMixer * mix)
   mix->state_lock = g_mutex_new ();
   /* initialize variables */
   gst_videomixer_reset (mix);
-
 }
 
 static void
@@ -742,11 +655,11 @@ gst_videomixer_request_new_pad (GstElement * element,
 
   /* add the pad to the element */
   gst_element_add_pad (element, GST_PAD (mixpad));
+  gst_child_proxy_child_added (GST_OBJECT (mix), GST_OBJECT (mixpad));
 
   return GST_PAD (mixpad);
 }
 
-
 static void
 gst_videomixer_release_pad (GstElement * element, GstPad * pad)
 {
@@ -765,6 +678,7 @@ gst_videomixer_release_pad (GstElement * element, GstPad * pad)
   mix->sinkpads = g_slist_remove (mix->sinkpads, pad);
   gst_videomixer_collect_free (mixpad->mixcol);
   gst_collect_pads_remove_pad (mix->collect, pad);
+  gst_child_proxy_child_removed (GST_OBJECT (mix), GST_OBJECT (mixpad));
   /* determine possibly new geometry and master */
   gst_videomixer_set_master_geometry (mix);
   mix->numpads--;
@@ -924,10 +838,6 @@ error:
 #define BLEND_MODE BLEND_HARDLIGHT
 #endif
 
-#define ROUND_UP_2(x) (((x) + 1) & ~1)
-#define ROUND_UP_4(x) (((x) + 3) & ~3)
-#define ROUND_UP_8(x) (((x) + 7) & ~7)
-
 /* note that this function does packing conversion and blending at the
  * same time */
 static void
@@ -941,8 +851,8 @@ gst_videomixer_blend_ayuv_ayuv (guint8 * src, gint xpos, gint ypos,
   gint src_add, dest_add;
   gint Y, U, V;
 
-  src_stride = ROUND_UP_2 (src_width) * 4;
-  dest_stride = ROUND_UP_2 (dest_width) * 4;
+  src_stride = src_width * 4;
+  dest_stride = dest_width * 4;
 
   b_alpha = (gint) (src_alpha * 255);
 
@@ -965,14 +875,14 @@ gst_videomixer_blend_ayuv_ayuv (guint8 * src, gint xpos, gint ypos,
     src_height = dest_height - ypos;
   }
 
-  src_add = src_stride - (4 * ROUND_UP_2 (src_width));
-  dest_add = dest_stride - (4 * ROUND_UP_2 (src_width));
+  src_add = src_stride - (4 * src_width);
+  dest_add = dest_stride - (4 * src_width);
 
   dest = dest + 4 * xpos + (ypos * dest_stride);
 
   /* we convert a square of 2x2 samples to generate 4 Luma and 2 chroma samples */
-  for (i = 0; i < ROUND_UP_2 (src_height); i++) {
-    for (j = 0; j < ROUND_UP_2 (src_width); j++) {
+  for (i = 0; i < src_height; i++) {
+    for (j = 0; j < src_width; j++) {
       alpha = (src[0] * b_alpha) >> 8;
       BLEND_MODE (dest[1], dest[2], dest[3], src[1], src[2], src[3],
           alpha, Y, U, V);
@@ -1009,14 +919,11 @@ gst_videomixer_sort_pads (GstVideoMixer * mix)
 static void
 gst_videomixer_fill_checker (guint8 * dest, gint width, gint height)
 {
-  gint stride;
   gint i, j;
   static int tab[] = { 80, 160, 80, 160 };
 
-  stride = ROUND_UP_2 (width);
-
   for (i = 0; i < height; i++) {
-    for (j = 0; j < stride; j++) {
+    for (j = 0; j < width; j++) {
       *dest++ = 0xff;
       *dest++ = tab[((i & 0x8) >> 3) + ((j & 0x8) >> 3)];
       *dest++ = 128;
@@ -1029,13 +936,10 @@ static void
 gst_videomixer_fill_color (guint8 * dest, gint width, gint height,
     gint colY, gint colU, gint colV)
 {
-  gint stride;
   gint i, j;
 
-  stride = ROUND_UP_2 (width);
-
   for (i = 0; i < height; i++) {
-    for (j = 0; j < stride; j++) {
+    for (j = 0; j < width; j++) {
       *dest++ = 0xff;
       *dest++ = colY;
       *dest++ = colU;
@@ -1093,6 +997,17 @@ gst_videomixer_fill_queues (GstVideoMixer * mix)
         GST_LOG ("pop returned a NULL buffer");
       }
     }
+    if (mix->sendseg && (mixpad == mix->master)) {
+      GstEvent *event;
+      GstSegment *segment = &data->segment;
+
+      GST_INFO ("_sending play segment");
+      event = gst_event_new_new_segment (FALSE, segment->rate, segment->format,
+          segment->start, segment->stop, segment->time);
+      gst_pad_push_event (mix->srcpad, event);
+      mix->sendseg = FALSE;
+    }
+
     if (mixcol->buffer != NULL && GST_CLOCK_TIME_IS_VALID (mixpad->queued)) {
       /* got a buffer somewhere so we're not eos */
       eos = FALSE;
@@ -1116,10 +1031,10 @@ gst_videomixer_blend_buffers (GstVideoMixer * mix, GstBuffer * outbuf)
     walk = g_slist_next (walk);
 
     if (mixcol->buffer != NULL) {
+      gst_object_sync_values (G_OBJECT (pad),
+          GST_BUFFER_TIMESTAMP (mixcol->buffer));
       gst_videomixer_blend_ayuv_ayuv (GST_BUFFER_DATA (mixcol->buffer),
-          pad->xpos, pad->ypos,
-          pad->in_width, pad->in_height,
-          pad->alpha,
+          pad->xpos, pad->ypos, pad->in_width, pad->in_height, pad->alpha,
           GST_BUFFER_DATA (outbuf), mix->out_width, mix->out_height);
       if (pad == mix->master) {
         GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (mixcol->buffer);
@@ -1190,7 +1105,7 @@ gst_videomixer_collected (GstCollectPads * pads, GstVideoMixer * mix)
   }
 
   /* Calculating out buffer size from input size */
-  outsize = ROUND_UP_2 (mix->in_width) * ROUND_UP_2 (mix->in_height) * 4;
+  outsize = 4 * mix->in_width * GST_ROUND_UP_2 (mix->in_height);
 
   /* If geometry has changed we need to set new caps on the buffer */
   if (mix->in_width != mix->out_width || mix->in_height != mix->out_height
