@@ -20,6 +20,37 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-osssink
+ * @short_description: output sound using OSS
+ *
+ * <refsect2>
+ * <para>
+ * This element lets you output sound using the Open Sound System (OSS).
+ * </para>
+ * <para>
+ * Note that you should almost always use generic audio conversion elements
+ * like audioconvert and audioresample in front of an audiosink to make sure
+ * your pipeline works under all circumstances (those conversion elements will
+ * act in passthrough-mode if no conversion is necessary).
+ * </para>
+ * <title>Example pipelines</title>
+ * <para>
+ * <programlisting>
+ * gst-launch -v audiotestsrc ! audioconvert ! volume volume=0.1 ! osssink
+ * </programlisting>
+ * will output a sine wave (continuous beep sound) to your sound card (with
+ * a very low volume as precaution).
+ * </para>
+ * <para>
+ * <programlisting>
+ * gst-launch -v filesrc location=music.ogg ! decodebin ! audioconvert ! audioresample ! osssink
+ * </programlisting>
+ * will play an Ogg/Vorbis audio file and output it using the Open Sound System.
+ * </para>
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -46,6 +77,8 @@
 #include "common.h"
 #include "gstosssink.h"
 
+#include <gst/gst-i18n-plugin.h>
+
 GST_DEBUG_CATEGORY_EXTERN (oss_debug);
 #define GST_CAT_DEFAULT oss_debug
 
@@ -204,9 +237,14 @@ gst_oss_sink_class_init (GstOssSinkClass * klass)
 static void
 gst_oss_sink_init (GstOssSink * osssink)
 {
+  const gchar *device;
+
   GST_DEBUG_OBJECT (osssink, "initializing osssink");
 
-  osssink->device = g_strdup (DEFAULT_DEVICE);
+  device = g_getenv ("AUDIODEV");
+  if (device == NULL)
+    device = DEFAULT_DEVICE;
+  osssink->device = g_strdup (device);
   osssink->fd = -1;
 }
 
@@ -216,6 +254,8 @@ gst_oss_sink_finalise (GObject * object)
   GstOssSink *osssink = GST_OSSSINK (object);
 
   g_free (osssink->device);
+
+  G_OBJECT_CLASS (parent_class)->finalize ((GObject *) (object));
 }
 
 static void
@@ -358,6 +398,8 @@ gst_oss_sink_open (GstAudioSink * asink)
     switch (errno) {
       case EBUSY:
         goto busy;
+      case EACCES:
+        goto no_permission;
       default:
         goto open_failed;
     }
@@ -365,15 +407,26 @@ gst_oss_sink_open (GstAudioSink * asink)
 
   return TRUE;
 
+  /* ERRORS */
 busy:
   {
-    GST_ELEMENT_ERROR (oss, RESOURCE, BUSY, (NULL), (NULL));
+    GST_ELEMENT_ERROR (oss, RESOURCE, BUSY,
+        (_("Could not open audio device for playback. "
+                "Device is being used by another application.")), (NULL));
+    return FALSE;
+  }
+no_permission:
+  {
+    GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_WRITE,
+        (_("Could not open audio device for playback."
+                "You don't have permission to open the device.")),
+        GST_ERROR_SYSTEM);
     return FALSE;
   }
-
 open_failed:
   {
-    GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_WRITE, (NULL), GST_ERROR_SYSTEM);
+    GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_WRITE,
+        (_("Could not open audio device for playback.")), GST_ERROR_SYSTEM);
     return FALSE;
   }
 }
@@ -396,10 +449,17 @@ gst_oss_sink_prepare (GstAudioSink * asink, GstRingBufferSpec * spec)
 
   oss = GST_OSSSINK (asink);
 
+  /* we opened non-blocking so that we can detect if the device is available
+   * without hanging forever. We now want to remove the non-blocking flag. */
   mode = fcntl (oss->fd, F_GETFL);
   mode &= ~O_NONBLOCK;
-  if (fcntl (oss->fd, F_SETFL, mode) == -1)
-    goto non_block;
+  if (fcntl (oss->fd, F_SETFL, mode) == -1) {
+    /* some drivers do no support unsetting the non-blocking flag, try to
+     * close/open the device then. This is racy but we error out properly. */
+    gst_oss_sink_close (asink);
+    if ((oss->fd = open (oss->device, O_WRONLY, 0)) == -1)
+      goto non_block;
+  }
 
   tmp = gst_oss_sink_get_format (spec->format);
   if (tmp == 0)
@@ -427,13 +487,13 @@ gst_oss_sink_prepare (GstAudioSink * asink, GstRingBufferSpec * spec)
 
   spec->bytes_per_sample = (spec->width / 8) * spec->channels;
   oss->bytes_per_sample = (spec->width / 8) * spec->channels;
-  memset (spec->silence_sample, 0, spec->bytes_per_sample);
 
   GST_DEBUG_OBJECT (oss, "got segsize: %d, segtotal: %d, value: %08x",
       spec->segsize, spec->segtotal, tmp);
 
   return TRUE;
 
+  /* ERRORS */
 non_block:
   {
     GST_ELEMENT_ERROR (oss, RESOURCE, SETTINGS, (NULL),
@@ -468,6 +528,7 @@ gst_oss_sink_unprepare (GstAudioSink * asink)
 
   return TRUE;
 
+  /* ERRORS */
 couldnt_close:
   {
     GST_DEBUG_OBJECT (asink, "Could not close the audio device");
