@@ -1,6 +1,6 @@
 /*
- * GStreamer
- * Copyright (C) 2005 Brian Cameron <brian.cameron@sun.com>
+ * GStreamer - SunAudio mixer interface element
+ * Copyright (C) 2005,2006 Brian Cameron <brian.cameron@sun.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -29,12 +29,16 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/ioctl.h>
+#include <sys/mixer.h>
 
 #include <gst/gst-i18n-plugin.h>
 
 #include "gstsunaudiomixerctrl.h"
 #include "gstsunaudiomixertrack.h"
 
+GST_DEBUG_CATEGORY_EXTERN (sunaudio_debug);
+#define GST_CAT_DEFAULT sunaudio_debug
+
 #define SCALE_FACTOR 2.55       /* 255/100 */
 
 static gboolean
@@ -57,6 +61,8 @@ gst_sunaudiomixer_ctrl_open (GstSunAudioMixerCtrl * mixer)
 
     return FALSE;
   }
+  /* Try to set the multiple open flag if we can, but ignore errors */
+  ioctl (mixer->mixer_fd, AUDIO_MIXER_MULTIPLE_OPEN);
 
   mixer->mixer_fd = fd;
   return TRUE;
@@ -74,9 +80,9 @@ gst_sunaudiomixer_ctrl_build_list (GstSunAudioMixerCtrl * mixer)
   if (mixer->tracklist == NULL) {
     g_return_if_fail (mixer->mixer_fd != -1);
 
-    /* Output */
+    /* Output & should be MASTER when it's the only one. */
     track = gst_sunaudiomixer_track_new (GST_SUNAUDIO_TRACK_OUTPUT,
-        2, GST_MIXER_TRACK_OUTPUT);
+        2, GST_MIXER_TRACK_OUTPUT | GST_MIXER_TRACK_MASTER);
     mixer->tracklist = g_list_append (mixer->tracklist, track);
 
     /* Input */
