@@ -37,10 +37,14 @@
  * errors here, check your linux/time.h && sys/time.h header setup.
  */
 #include <sys/types.h>
+#ifndef __sun
 #include <linux/types.h>
 #define _LINUX_TIME_H
 #define __user
 #include <linux/videodev2.h>
+#else
+#include <sys/videodev2.h>
+#endif
 
 #include <gst/gst.h>
 #include <gst/base/gstpushsrc.h>
@@ -105,23 +109,19 @@ struct _GstV4l2ObjectClassHelper {
   GList *devices;
 };
 
-GType gst_v4l2_object_get_type(void);
-
-#define V4L2_STD_OBJECT_PROPS			\
-    PROP_DEVICE,				\
-    PROP_DEVICE_NAME,				\
-    PROP_FLAGS,					\
-    PROP_NORM,					\
-    PROP_CHANNEL,					\
-    PROP_FREQUENCY
+GType gst_v4l2_object_get_type (void);
 
+#define V4L2_STD_OBJECT_PROPS		\
+    PROP_DEVICE,			\
+    PROP_DEVICE_NAME,			\
+    PROP_FLAGS
 
 /* create/destroy */
 GstV4l2Object *	gst_v4l2_object_new 		 (GstElement * element,
                    				  GstV4l2GetInOutFunction get_in_out_func,
                    				  GstV4l2SetInOutFunction set_in_out_func,
 		   				  GstV4l2UpdateFpsFunction   update_fps_func);
-void 	        gst_v4l2_object_destroy 	 (GstV4l2Object ** v4l2object);
+void 	        gst_v4l2_object_destroy 	 (GstV4l2Object * v4l2object);
 
 /* properties */
 void 	  gst_v4l2_object_install_properties_helper (GObjectClass *gobject_class);
